----------------------------------------------------
--
--  Library Name :  CTUasVTU
--  Unit    Name :  ctu_pckg
--  Unit    Type :  Package
--
------------------------------------------------------
------------------------------------------
------------------------------------------
-- Date        : November 14, 2014
--
-- Author      : G. Hagmann
--
-- Company     : CERN BE-RF-FB
--
-- Description : CTU package
--
-- Changelog   : 20161019 (1.1)
--                 Modified the reduced or for proper synthesis
--
--               20150227 (1.0)
--                 Initial revision.
------------------------------------------
------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package ctu_Pckg is

constant C_FirmwareVersionInt : integer := 20170306;

-- Firmware and Memory Map version codes
constant C_FirmwareVersion  : Std_logic_vector(31 downto 0) := std_logic_vector(to_unsigned(C_FirmwareVersionInt,32));

-- NVMem data size in 16b words
constant c_nvMemCalibDataSize     : Std_logic_vector(13 downto 0) := "00"& X"004";  -- 4 words: rfPowerGain, rfPowerOffs, sync1IDelay, driveOutODelay

-- Function or_reduce
function or_reduce( V: std_logic_vector ) return std_logic;

-- Function clean_data
function clean_data( V: std_logic_vector(7 downto 0) ) return std_logic_vector;

end;



package body  ctu_pckg  is

  -- Function or_reduce
  function or_reduce( V: std_logic_vector ) return std_logic is
    variable result: std_logic;
  begin
    result := '0';
    for i in V'range loop
      result := result OR V(i);
    end loop;
    return result;
  end or_reduce;

  -- Function clean_data
  --   Combinatorial. Search for the first '1' and removes other '1's.
  function clean_data( V: std_logic_vector(7 downto 0) ) return std_logic_vector is
  begin
    if V(7)='1' then
      return "10000000";
    elsif V(6)='1' then
      return "01000000";
    elsif V(5)='1' then
      return "00100000";
    elsif V(4)='1' then
      return "00010000";
    elsif V(3)='1' then
      return "00001000";
    elsif V(2)='1' then
      return "00000100";
    elsif V(1)='1' then
      return "00000010";
    elsif V(0)='1' then
      return "00000001";
    else
      return "00000000";
    end if;
  end clean_data;
end;

----------------------------------------------------
--
--  Library Name :  CTUasVTU
--  Unit    Name :  MemMap_ctuAsVtu
--  Unit    Type :  Package
--
------------------------------------------------------

-----------------------------------------------
-----------------------------------------------
-- Date          : Fri Feb 24 15:12:20 2017
--
-- Author        : dbarrien
--
-- Company       : CERN, BE-RF
--
-- Description   : VTU functionality in Chopper Trigger Unit (CTU)
--
-- Changelog     : 20170224 Initial version
--
-----------------------------------------------
-----------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.NUMERIC_STD.all;


-- MemoryMap : EDA-03072.
package MemMap_ctuAsVtu is

	-- Ident Code
	constant C_ctuAsVtu_IdentCode : std_logic_vector(15 downto 0) := X"00ca";

	-- Memory Map Version
	constant C_ctuAsVtu_MemMapVersion : std_logic_vector(31 downto 0) := std_logic_vector(to_unsigned(20170224,32));

	-- Memory Areas.
	constant C_Area_ctuAsVtu_adcMonitoring  : std_logic_vector(19 downto 5) := "000000000010000";-- : Word Address : X"100"; Byte Address : 0x200
	constant C_Area_ctuAsVtu_ctrvVMESlot1   : std_logic_vector(19 downto 16) := "0101";--Reserved area : Word Address : X"28000"; Byte Address : 0x50000
	constant C_Area_ctuAsVtu_ctrvVMESlot2   : std_logic_vector(19 downto 16) := "0110";--Reserved area : Word Address : X"30000"; Byte Address : 0x60000
	constant C_Area_ctuAsVtu_ctrvVMESlot3   : std_logic_vector(19 downto 16) := "0111";--Reserved area : Word Address : X"38000"; Byte Address : 0x70000
	-- Register Addresses : Area adcMonitoring
	constant C_Reg_ctuAsVtu_adcMonitoring_uP3V3D : std_logic_vector(4 downto 1) := "0000";-- : Word address : 0x0; Byte Address : 0x0
	constant C_Reg_ctuAsVtu_adcMonitoring_uM6VA : std_logic_vector(4 downto 1) := "0001";-- : Word address : 0x1; Byte Address : 0x2
	constant C_Reg_ctuAsVtu_adcMonitoring_uP12V : std_logic_vector(4 downto 1) := "0010";-- : Word address : 0x2; Byte Address : 0x4
	constant C_Reg_ctuAsVtu_adcMonitoring_uM12V : std_logic_vector(4 downto 1) := "0011";-- : Word address : 0x3; Byte Address : 0x6
	constant C_Reg_ctuAsVtu_adcMonitoring_uM5VA : std_logic_vector(4 downto 1) := "0100";-- : Word address : 0x4; Byte Address : 0x8
	constant C_Reg_ctuAsVtu_adcMonitoring_uP5V : std_logic_vector(4 downto 1) := "0101";-- : Word address : 0x5; Byte Address : 0xa
	constant C_Reg_ctuAsVtu_adcMonitoring_uVTT : std_logic_vector(4 downto 1) := "0110";-- : Word address : 0x6; Byte Address : 0xc
	constant C_Reg_ctuAsVtu_adcMonitoring_uP3V3 : std_logic_vector(4 downto 1) := "0111";-- : Word address : 0x7; Byte Address : 0xe
	constant C_Reg_ctuAsVtu_adcMonitoring_uP1V8 : std_logic_vector(4 downto 1) := "1000";-- : Word address : 0x8; Byte Address : 0x10
	constant C_Reg_ctuAsVtu_adcMonitoring_uP1V0 : std_logic_vector(4 downto 1) := "1001";-- : Word address : 0x9; Byte Address : 0x12
	-- Register Auto Clear Masks : Area adcMonitoring
	constant C_ACM_ctuAsVtu_adcMonitoring_uP3V3D        : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uM6VA         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uP12V         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uM12V         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uM5VA         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uP5V          : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uVTT          : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uP3V3         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uP1V8         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_adcMonitoring_uP1V0         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	-- Register Preset Masks : Area adcMonitoring
	constant C_PSM_ctuAsVtu_adcMonitoring_uP3V3D        : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uM6VA         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uP12V         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uM12V         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uM5VA         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uP5V          : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uVTT          : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uP3V3         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uP1V8         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_adcMonitoring_uP1V0         : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	-- Register Addresses
	constant C_Reg_ctuAsVtu_control1           : std_logic_vector(19 downto 1) := "0000000000000000000";--control1 word layout : Word address : "000" & X"0000"; Byte Address : 0x0
	constant C_Reg_ctuAsVtu_vmeIRQStatID       : std_logic_vector(19 downto 1) := "0000000000000000001";-- : Word address : "000" & X"0001"; Byte Address : 0x2
	constant C_Reg_ctuAsVtu_vmeIRQLevel        : std_logic_vector(19 downto 1) := "0000000000000000010";-- : Word address : "000" & X"0002"; Byte Address : 0x4
	constant C_Reg_ctuAsVtu_status1            : std_logic_vector(19 downto 1) := "0000000000000000011";-- : Word address : "000" & X"0003"; Byte Address : 0x6
	constant C_Reg_ctuAsVtu_ident              : std_logic_vector(19 downto 1) := "0000000000000000100";--Card ID = 0x00CA (Linac4) : Word address : "000" & X"0004"; Byte Address : 0x8
	constant C_Reg_ctuAsVtu_faults1            : std_logic_vector(19 downto 1) := "0000000000000000101";--Faults bits are static. To clear fault, use ClearFaults bit in control1 register : Word address : "000" & X"0005"; Byte Address : 0xa
	constant C_Reg_ctuAsVtu_overflow1          : std_logic_vector(19 downto 1) := "0000000000000000110";-- : Word address : "000" & X"0006"; Byte Address : 0xc
	constant C_Reg_ctuAsVtu_nvMemStatus        : std_logic_vector(19 downto 1) := "0000000000000000111";-- : Word address : "000" & X"0007"; Byte Address : 0xe
	constant C_Reg_ctuAsVtu_nvMemControl       : std_logic_vector(19 downto 1) := "0000000000000001000";-- : Word address : "000" & X"0008"; Byte Address : 0x10
	constant C_Reg_ctuAsVtu_nvMemGetKey        : std_logic_vector(19 downto 1) := "0000000000000001001";-- : Word address : "000" & X"0009"; Byte Address : 0x12
	constant C_Reg_ctuAsVtu_nvMemSetKey        : std_logic_vector(19 downto 1) := "0000000000000001010";-- : Word address : "000" & X"000A"; Byte Address : 0x14
	constant C_Reg_ctuAsVtu_nvMemCRCVal        : std_logic_vector(19 downto 1) := "0000000000000001011";-- : Word address : "000" & X"000B"; Byte Address : 0x16
	constant C_Reg_ctuAsVtu_reset              : std_logic_vector(19 downto 1) := "0000000000000001100";-- : Word address : "000" & X"000C"; Byte Address : 0x18
	constant C_Reg_ctuAsVtu_inoutControl       : std_logic_vector(19 downto 1) := "0000000000000001101";-- : Word address : "000" & X"000D"; Byte Address : 0x1a
	constant C_Reg_ctuAsVtu_clockStatus        : std_logic_vector(19 downto 1) := "0000000000000001110";-- : Word address : "000" & X"000E"; Byte Address : 0x1c
	constant C_Reg_ctuAsVtu_clockFaults        : std_logic_vector(19 downto 1) := "0000000000000001111";-- : Word address : "000" & X"000F"; Byte Address : 0x1e
	constant C_Reg_ctuAsVtu_freqWindows        : std_logic_vector(19 downto 1) := "0000000000000010000";-- : Word address : "000" & X"0010"; Byte Address : 0x20
	constant C_Reg_ctuAsVtu_bpECL1Freq_1       : std_logic_vector(19 downto 1) := "0000000000000010001";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0011"; Byte Address : 0x22
	constant C_Reg_ctuAsVtu_bpECL1Freq_0       : std_logic_vector(19 downto 1) := "0000000000000010010";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0012"; Byte Address : 0x24
	constant C_Reg_ctuAsVtu_bpECL2Freq_1       : std_logic_vector(19 downto 1) := "0000000000000010011";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0013"; Byte Address : 0x26
	constant C_Reg_ctuAsVtu_bpECL2Freq_0       : std_logic_vector(19 downto 1) := "0000000000000010100";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0014"; Byte Address : 0x28
	constant C_Reg_ctuAsVtu_bpECL3Freq_1       : std_logic_vector(19 downto 1) := "0000000000000010101";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0015"; Byte Address : 0x2a
	constant C_Reg_ctuAsVtu_bpECL3Freq_0       : std_logic_vector(19 downto 1) := "0000000000000010110";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0016"; Byte Address : 0x2c
	constant C_Reg_ctuAsVtu_bpECL4Freq_1       : std_logic_vector(19 downto 1) := "0000000000000010111";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0017"; Byte Address : 0x2e
	constant C_Reg_ctuAsVtu_bpECL4Freq_0       : std_logic_vector(19 downto 1) := "0000000000000011000";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0018"; Byte Address : 0x30
	constant C_Reg_ctuAsVtu_rfFreq_1           : std_logic_vector(19 downto 1) := "0000000000000011001";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0019"; Byte Address : 0x32
	constant C_Reg_ctuAsVtu_rfFreq_0           : std_logic_vector(19 downto 1) := "0000000000000011010";--Conversion factor changes according to freqWindows register : Word address : "000" & X"001A"; Byte Address : 0x34
	constant C_Reg_ctuAsVtu_rfFreqMax_1        : std_logic_vector(19 downto 1) := "0000000000000011011";--Conversion factor changes according to freqWindows register : Word address : "000" & X"001B"; Byte Address : 0x36
	constant C_Reg_ctuAsVtu_rfFreqMax_0        : std_logic_vector(19 downto 1) := "0000000000000011100";--Conversion factor changes according to freqWindows register : Word address : "000" & X"001C"; Byte Address : 0x38
	constant C_Reg_ctuAsVtu_rfFreqMin_1        : std_logic_vector(19 downto 1) := "0000000000000011101";--Conversion factor changes according to freqWindows register : Word address : "000" & X"001D"; Byte Address : 0x3a
	constant C_Reg_ctuAsVtu_rfFreqMin_0        : std_logic_vector(19 downto 1) := "0000000000000011110";--Conversion factor changes according to freqWindows register : Word address : "000" & X"001E"; Byte Address : 0x3c
	constant C_Reg_ctuAsVtu_rfDiv1Freq_1       : std_logic_vector(19 downto 1) := "0000000000000011111";--Conversion factor changes according to freqWindows register : Word address : "000" & X"001F"; Byte Address : 0x3e
	constant C_Reg_ctuAsVtu_rfDiv1Freq_0       : std_logic_vector(19 downto 1) := "0000000000000100000";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0020"; Byte Address : 0x40
	constant C_Reg_ctuAsVtu_rfDiv2Freq_1       : std_logic_vector(19 downto 1) := "0000000000000100001";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0021"; Byte Address : 0x42
	constant C_Reg_ctuAsVtu_rfDiv2Freq_0       : std_logic_vector(19 downto 1) := "0000000000000100010";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0022"; Byte Address : 0x44
	constant C_Reg_ctuAsVtu_syncFreq_1         : std_logic_vector(19 downto 1) := "0000000000000100011";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0023"; Byte Address : 0x46
	constant C_Reg_ctuAsVtu_syncFreq_0         : std_logic_vector(19 downto 1) := "0000000000000100100";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0024"; Byte Address : 0x48
	constant C_Reg_ctuAsVtu_syncFreqMax_1      : std_logic_vector(19 downto 1) := "0000000000000100101";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0025"; Byte Address : 0x4a
	constant C_Reg_ctuAsVtu_syncFreqMax_0      : std_logic_vector(19 downto 1) := "0000000000000100110";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0026"; Byte Address : 0x4c
	constant C_Reg_ctuAsVtu_syncFreqMin_1      : std_logic_vector(19 downto 1) := "0000000000000100111";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0027"; Byte Address : 0x4e
	constant C_Reg_ctuAsVtu_syncFreqMin_0      : std_logic_vector(19 downto 1) := "0000000000000101000";--Conversion factor changes according to freqWindows register : Word address : "000" & X"0028"; Byte Address : 0x50
	constant C_Reg_ctuAsVtu_pdcSyncDelay       : std_logic_vector(19 downto 1) := "0000000000000101001";-- : Word address : "000" & X"0029"; Byte Address : 0x52
	constant C_Reg_ctuAsVtu_pdcOutDelay        : std_logic_vector(19 downto 1) := "0000000000000101010";-- : Word address : "000" & X"002A"; Byte Address : 0x54
	constant C_Reg_ctuAsVtu_refPhase           : std_logic_vector(19 downto 1) := "0000000000000101011";-- : Word address : "000" & X"002B"; Byte Address : 0x56
	constant C_Reg_ctuAsVtu_rfPowerMax         : std_logic_vector(19 downto 1) := "0000000000000101100";-- : Word address : "000" & X"002C"; Byte Address : 0x58
	constant C_Reg_ctuAsVtu_rfPowerMin         : std_logic_vector(19 downto 1) := "0000000000000101101";-- : Word address : "000" & X"002D"; Byte Address : 0x5a
	constant C_Reg_ctuAsVtu_iodelayControl     : std_logic_vector(19 downto 1) := "0000000000000101110";-- : Word address : "000" & X"002E"; Byte Address : 0x5c
	constant C_Reg_ctuAsVtu_iodelayStatus      : std_logic_vector(19 downto 1) := "0000000000000101111";-- : Word address : "000" & X"002F"; Byte Address : 0x5e
	constant C_Reg_ctuAsVtu_ispPowrVersion_1   : std_logic_vector(19 downto 1) := "0000000000000110000";-- : Word address : "000" & X"0030"; Byte Address : 0x60
	constant C_Reg_ctuAsVtu_ispPowrVersion_0   : std_logic_vector(19 downto 1) := "0000000000000110001";-- : Word address : "000" & X"0031"; Byte Address : 0x62
	constant C_Reg_ctuAsVtu_triggerSelector    : std_logic_vector(19 downto 1) := "0000000000000110010";-- : Word address : "000" & X"0032"; Byte Address : 0x64
	constant C_Reg_ctuAsVtu_control2           : std_logic_vector(19 downto 1) := "0000000000000110011";-- : Word address : "000" & X"0033"; Byte Address : 0x66
	constant C_Reg_ctuAsVtu_xadcStatus         : std_logic_vector(19 downto 1) := "0000000000000110100";-- : Word address : "000" & X"0034"; Byte Address : 0x68
	constant C_Reg_ctuAsVtu_xadcAlarms         : std_logic_vector(19 downto 1) := "0000000000000110101";-- : Word address : "000" & X"0035"; Byte Address : 0x6a
	constant C_Reg_ctuAsVtu_timingControl      : std_logic_vector(19 downto 1) := "0000000000000110110";-- : Word address : "000" & X"0036"; Byte Address : 0x6c
	constant C_Reg_ctuAsVtu_timingStatus       : std_logic_vector(19 downto 1) := "0000000000000110111";-- : Word address : "000" & X"0037"; Byte Address : 0x6e
	constant C_Reg_ctuAsVtu_interlock11ASel    : std_logic_vector(19 downto 1) := "0000000000000111000";-- : Word address : "000" & X"0038"; Byte Address : 0x70
	constant C_Reg_ctuAsVtu_interlock11BSel    : std_logic_vector(19 downto 1) := "0000000000000111001";-- : Word address : "000" & X"0039"; Byte Address : 0x72
	constant C_Reg_ctuAsVtu_playMemControl_1   : std_logic_vector(19 downto 1) := "0000000000000111010";-- : Word address : "000" & X"003A"; Byte Address : 0x74
	constant C_Reg_ctuAsVtu_playMemControl_0   : std_logic_vector(19 downto 1) := "0000000000000111011";-- : Word address : "000" & X"003B"; Byte Address : 0x76
	constant C_Reg_ctuAsVtu_sysControl         : std_logic_vector(19 downto 1) := "0000000000001110000";-- : Word address : "000" & X"0070"; Byte Address : 0xe0
	constant C_Reg_ctuAsVtu_hardwareVersion    : std_logic_vector(19 downto 1) := "0000000000001110001";-- : Word address : "000" & X"0071"; Byte Address : 0xe2
	constant C_Reg_ctuAsVtu_designerID         : std_logic_vector(19 downto 1) := "0000000000001110010";-- : Word address : "000" & X"0072"; Byte Address : 0xe4
	constant C_Reg_ctuAsVtu_boardTemp          : std_logic_vector(19 downto 1) := "0000000000001110011";-- : Word address : "000" & X"0073"; Byte Address : 0xe6
	constant C_Reg_ctuAsVtu_reserved1          : std_logic_vector(19 downto 1) := "0000000000001110101";-- : Word address : "000" & X"0075"; Byte Address : 0xea
	constant C_Reg_ctuAsVtu_testControl        : std_logic_vector(19 downto 1) := "0000000000001110110";-- : Word address : "000" & X"0076"; Byte Address : 0xec
	constant C_Reg_ctuAsVtu_testStatus         : std_logic_vector(19 downto 1) := "0000000000001110111";-- : Word address : "000" & X"0077"; Byte Address : 0xee
	constant C_Reg_ctuAsVtu_firmwareVersion_1  : std_logic_vector(19 downto 1) := "0000000000001111000";-- : Word address : "000" & X"0078"; Byte Address : 0xf0
	constant C_Reg_ctuAsVtu_firmwareVersion_0  : std_logic_vector(19 downto 1) := "0000000000001111001";-- : Word address : "000" & X"0079"; Byte Address : 0xf2
	constant C_Reg_ctuAsVtu_serialNumber_3     : std_logic_vector(19 downto 1) := "0000000000001111010";-- : Word address : "000" & X"007A"; Byte Address : 0xf4
	constant C_Reg_ctuAsVtu_serialNumber_2     : std_logic_vector(19 downto 1) := "0000000000001111011";-- : Word address : "000" & X"007B"; Byte Address : 0xf6
	constant C_Reg_ctuAsVtu_serialNumber_1     : std_logic_vector(19 downto 1) := "0000000000001111100";-- : Word address : "000" & X"007C"; Byte Address : 0xf8
	constant C_Reg_ctuAsVtu_serialNumber_0     : std_logic_vector(19 downto 1) := "0000000000001111101";-- : Word address : "000" & X"007D"; Byte Address : 0xfa
	constant C_Reg_ctuAsVtu_memMapVersion_1    : std_logic_vector(19 downto 1) := "0000000000001111110";-- : Word address : "000" & X"007E"; Byte Address : 0xfc
	constant C_Reg_ctuAsVtu_memMapVersion_0    : std_logic_vector(19 downto 1) := "0000000000001111111";-- : Word address : "000" & X"007F"; Byte Address : 0xfe
	constant C_Reg_ctuAsVtu_rfPowerGain        : std_logic_vector(19 downto 1) := "0000000000010000000";-- : Word address : "000" & X"0080"; Byte Address : 0x100
	constant C_Reg_ctuAsVtu_rfPowerOffs        : std_logic_vector(19 downto 1) := "0000000000010000001";-- : Word address : "000" & X"0081"; Byte Address : 0x102
	constant C_Reg_ctuAsVtu_syncIDelay         : std_logic_vector(19 downto 1) := "0000000000010000010";-- : Word address : "000" & X"0082"; Byte Address : 0x104
	constant C_Reg_ctuAsVtu_outODelay          : std_logic_vector(19 downto 1) := "0000000000010000011";-- : Word address : "000" & X"0083"; Byte Address : 0x106
	constant C_Reg_ctuAsVtu_bValueOffline_3    : std_logic_vector(19 downto 1) := "0000000000010000100";--Used in all modes : Word address : "000" & X"0084"; Byte Address : 0x108
	constant C_Reg_ctuAsVtu_bValueOffline_2    : std_logic_vector(19 downto 1) := "0000000000010000101";--Used in all modes : Word address : "000" & X"0085"; Byte Address : 0x10a
	constant C_Reg_ctuAsVtu_bValueOffline_1    : std_logic_vector(19 downto 1) := "0000000000010000110";--Used in all modes : Word address : "000" & X"0086"; Byte Address : 0x10c
	constant C_Reg_ctuAsVtu_bValueOffline_0    : std_logic_vector(19 downto 1) := "0000000000010000111";--Used in all modes : Word address : "000" & X"0087"; Byte Address : 0x10e
	constant C_Reg_ctuAsVtu_htValueOffline_3   : std_logic_vector(19 downto 1) := "0000000000010001000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"0088"; Byte Address : 0x110
	constant C_Reg_ctuAsVtu_htValueOffline_2   : std_logic_vector(19 downto 1) := "0000000000010001001";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"0089"; Byte Address : 0x112
	constant C_Reg_ctuAsVtu_htValueOffline_1   : std_logic_vector(19 downto 1) := "0000000000010001010";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"008A"; Byte Address : 0x114
	constant C_Reg_ctuAsVtu_htValueOffline_0   : std_logic_vector(19 downto 1) := "0000000000010001011";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"008B"; Byte Address : 0x116
	constant C_Reg_ctuAsVtu_wValueOffline_3    : std_logic_vector(19 downto 1) := "0000000000010001100";--Used in windowedOperation mode : Word address : "000" & X"008C"; Byte Address : 0x118
	constant C_Reg_ctuAsVtu_wValueOffline_2    : std_logic_vector(19 downto 1) := "0000000000010001101";--Used in windowedOperation mode : Word address : "000" & X"008D"; Byte Address : 0x11a
	constant C_Reg_ctuAsVtu_wValueOffline_1    : std_logic_vector(19 downto 1) := "0000000000010001110";--Used in windowedOperation mode : Word address : "000" & X"008E"; Byte Address : 0x11c
	constant C_Reg_ctuAsVtu_wValueOffline_0    : std_logic_vector(19 downto 1) := "0000000000010001111";--Used in windowedOperation mode : Word address : "000" & X"008F"; Byte Address : 0x11e
	constant C_Reg_ctuAsVtu_bValueOnline_3     : std_logic_vector(19 downto 1) := "0000000000010010000";--Used in all modes : Word address : "000" & X"0090"; Byte Address : 0x120
	constant C_Reg_ctuAsVtu_bValueOnline_2     : std_logic_vector(19 downto 1) := "0000000000010010001";--Used in all modes : Word address : "000" & X"0091"; Byte Address : 0x122
	constant C_Reg_ctuAsVtu_bValueOnline_1     : std_logic_vector(19 downto 1) := "0000000000010010010";--Used in all modes : Word address : "000" & X"0092"; Byte Address : 0x124
	constant C_Reg_ctuAsVtu_bValueOnline_0     : std_logic_vector(19 downto 1) := "0000000000010010011";--Used in all modes : Word address : "000" & X"0093"; Byte Address : 0x126
	constant C_Reg_ctuAsVtu_htValueOnline_3    : std_logic_vector(19 downto 1) := "0000000000010010100";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"0094"; Byte Address : 0x128
	constant C_Reg_ctuAsVtu_htValueOnline_2    : std_logic_vector(19 downto 1) := "0000000000010010101";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"0095"; Byte Address : 0x12a
	constant C_Reg_ctuAsVtu_htValueOnline_1    : std_logic_vector(19 downto 1) := "0000000000010010110";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"0096"; Byte Address : 0x12c
	constant C_Reg_ctuAsVtu_htValueOnline_0    : std_logic_vector(19 downto 1) := "0000000000010010111";--Used in singlePulse, infiniteWindow and windowedOperation modes : Word address : "000" & X"0097"; Byte Address : 0x12e
	constant C_Reg_ctuAsVtu_wValueOnline_3     : std_logic_vector(19 downto 1) := "0000000000010011000";--Used in windowedOperation mode : Word address : "000" & X"0098"; Byte Address : 0x130
	constant C_Reg_ctuAsVtu_wValueOnline_2     : std_logic_vector(19 downto 1) := "0000000000010011001";--Used in windowedOperation mode : Word address : "000" & X"0099"; Byte Address : 0x132
	constant C_Reg_ctuAsVtu_wValueOnline_1     : std_logic_vector(19 downto 1) := "0000000000010011010";--Used in windowedOperation mode : Word address : "000" & X"009A"; Byte Address : 0x134
	constant C_Reg_ctuAsVtu_wValueOnline_0     : std_logic_vector(19 downto 1) := "0000000000010011011";--Used in windowedOperation mode : Word address : "000" & X"009B"; Byte Address : 0x136
	-- Register Auto Clear Masks : Memory Map
	constant C_ACM_ctuAsVtu_control1                    : std_logic_vector(7 downto 0) := "11101100";--control1 word layout : Value : X"EC"
	constant C_ACM_ctuAsVtu_vmeIRQStatID                : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_vmeIRQLevel                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_status1                     : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_ident                       : std_logic_vector(15 downto 0) := "0000000000000000";--Card ID = 0x00CA (Linac4) : Value : X"0000"
	constant C_ACM_ctuAsVtu_faults1                     : std_logic_vector(15 downto 0) := "0000000000000000";--Faults bits are static. To clear fault, use ClearFaults bit in control1 register : Value : X"0000"
	constant C_ACM_ctuAsVtu_overflow1                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_nvMemStatus                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_nvMemControl                : std_logic_vector(7 downto 0) := "00001111";-- : Value : X"0F"
	constant C_ACM_ctuAsVtu_nvMemGetKey                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_nvMemSetKey                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_nvMemCRCVal                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_reset                       : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_inoutControl                : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_clockStatus                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_clockFaults                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_freqWindows                 : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_ACM_ctuAsVtu_bpECL1Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bpECL1Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bpECL2Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bpECL2Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bpECL3Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bpECL3Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bpECL4Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bpECL4Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfFreq_1                    : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfFreq_0                    : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfFreqMax_1                 : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfFreqMax_0                 : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfFreqMin_1                 : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfFreqMin_0                 : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfDiv1Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfDiv1Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfDiv2Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfDiv2Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_syncFreq_1                  : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_syncFreq_0                  : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_syncFreqMax_1               : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_syncFreqMax_0               : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_syncFreqMin_1               : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_syncFreqMin_0               : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_ACM_ctuAsVtu_pdcSyncDelay                : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_pdcOutDelay                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_refPhase                    : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_rfPowerMax                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_rfPowerMin                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_iodelayControl              : std_logic_vector(7 downto 0) := "10000000";-- : Value : X"80"
	constant C_ACM_ctuAsVtu_iodelayStatus               : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_ispPowrVersion_1            : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_ispPowrVersion_0            : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_triggerSelector             : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_control2                    : std_logic_vector(7 downto 0) := "00000111";-- : Value : X"07"
	constant C_ACM_ctuAsVtu_xadcStatus                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_xadcAlarms                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_timingControl               : std_logic_vector(7 downto 0) := "10000000";-- : Value : X"80"
	constant C_ACM_ctuAsVtu_timingStatus                : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_interlock11ASel             : std_logic_vector(15 downto 0) := "1111110000011110";-- : Value : X"FC1E"
	constant C_ACM_ctuAsVtu_interlock11BSel             : std_logic_vector(15 downto 0) := "1111110000011110";-- : Value : X"FC1E"
	constant C_ACM_ctuAsVtu_playMemControl_1            : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_playMemControl_0            : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_sysControl                  : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_ACM_ctuAsVtu_hardwareVersion             : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_designerID                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_boardTemp                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_reserved1                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_testControl                 : std_logic_vector(7 downto 0) := "01110000";-- : Value : X"70"
	constant C_ACM_ctuAsVtu_testStatus                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_firmwareVersion_1           : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_firmwareVersion_0           : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_serialNumber_3              : std_logic_vector(63 downto 48) := "0000000000000000";-- : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_serialNumber_2              : std_logic_vector(47 downto 32) := "0000000000000000";-- : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_serialNumber_1              : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"000000000000"
	constant C_ACM_ctuAsVtu_serialNumber_0              : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_memMapVersion_1             : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_memMapVersion_0             : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_ACM_ctuAsVtu_rfPowerGain                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_rfPowerOffs                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_syncIDelay                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_outODelay                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_ACM_ctuAsVtu_bValueOffline_3             : std_logic_vector(63 downto 48) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_bValueOffline_2             : std_logic_vector(47 downto 32) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_bValueOffline_1             : std_logic_vector(31 downto 16) := "0000000000000000";--Used in all modes : Value : X"000000000000"
	constant C_ACM_ctuAsVtu_bValueOffline_0             : std_logic_vector(15 downto 0) := "0000000000000000";--Used in all modes : Value : X"00000000"
	constant C_ACM_ctuAsVtu_htValueOffline_3            : std_logic_vector(63 downto 48) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_htValueOffline_2            : std_logic_vector(47 downto 32) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_htValueOffline_1            : std_logic_vector(31 downto 16) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"000000000000"
	constant C_ACM_ctuAsVtu_htValueOffline_0            : std_logic_vector(15 downto 0) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"00000000"
	constant C_ACM_ctuAsVtu_wValueOffline_3             : std_logic_vector(63 downto 48) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_wValueOffline_2             : std_logic_vector(47 downto 32) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_wValueOffline_1             : std_logic_vector(31 downto 16) := "0000000000000000";--Used in windowedOperation mode : Value : X"000000000000"
	constant C_ACM_ctuAsVtu_wValueOffline_0             : std_logic_vector(15 downto 0) := "0000000000000000";--Used in windowedOperation mode : Value : X"00000000"
	constant C_ACM_ctuAsVtu_bValueOnline_3              : std_logic_vector(63 downto 48) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_bValueOnline_2              : std_logic_vector(47 downto 32) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_bValueOnline_1              : std_logic_vector(31 downto 16) := "0000000000000000";--Used in all modes : Value : X"000000000000"
	constant C_ACM_ctuAsVtu_bValueOnline_0              : std_logic_vector(15 downto 0) := "0000000000000000";--Used in all modes : Value : X"00000000"
	constant C_ACM_ctuAsVtu_htValueOnline_3             : std_logic_vector(63 downto 48) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_htValueOnline_2             : std_logic_vector(47 downto 32) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_htValueOnline_1             : std_logic_vector(31 downto 16) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"000000000000"
	constant C_ACM_ctuAsVtu_htValueOnline_0             : std_logic_vector(15 downto 0) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"00000000"
	constant C_ACM_ctuAsVtu_wValueOnline_3              : std_logic_vector(63 downto 48) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_wValueOnline_2              : std_logic_vector(47 downto 32) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_ACM_ctuAsVtu_wValueOnline_1              : std_logic_vector(31 downto 16) := "0000000000000000";--Used in windowedOperation mode : Value : X"000000000000"
	constant C_ACM_ctuAsVtu_wValueOnline_0              : std_logic_vector(15 downto 0) := "0000000000000000";--Used in windowedOperation mode : Value : X"00000000"
	-- Register Preset Masks : Memory Map
	constant C_PSM_ctuAsVtu_control1                    : std_logic_vector(7 downto 0) := "00000000";--control1 word layout : Value : X"00"
	constant C_PSM_ctuAsVtu_vmeIRQStatID                : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_vmeIRQLevel                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_status1                     : std_logic_vector(15 downto 0) := "1111111111111111";-- : Value : X"FFFF"
	constant C_PSM_ctuAsVtu_ident                       : std_logic_vector(15 downto 0) := "0000000000000000";--Card ID = 0x00CA (Linac4) : Value : X"0000"
	constant C_PSM_ctuAsVtu_faults1                     : std_logic_vector(15 downto 0) := "0000000000000000";--Faults bits are static. To clear fault, use ClearFaults bit in control1 register : Value : X"0000"
	constant C_PSM_ctuAsVtu_overflow1                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_nvMemStatus                 : std_logic_vector(15 downto 0) := "1111111111111111";-- : Value : X"FFFF"
	constant C_PSM_ctuAsVtu_nvMemControl                : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_PSM_ctuAsVtu_nvMemGetKey                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_nvMemSetKey                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_nvMemCRCVal                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_reset                       : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_inoutControl                : std_logic_vector(15 downto 0) := "1011100110000000";-- : Value : X"B980"
	constant C_PSM_ctuAsVtu_clockStatus                 : std_logic_vector(15 downto 0) := "1111111111111111";-- : Value : X"FFFF"
	constant C_PSM_ctuAsVtu_clockFaults                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_freqWindows                 : std_logic_vector(7 downto 0) := "00001101";-- : Value : X"0D"
	constant C_PSM_ctuAsVtu_bpECL1Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bpECL1Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bpECL2Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bpECL2Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bpECL3Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bpECL3Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bpECL4Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bpECL4Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_rfFreq_1                    : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_rfFreq_0                    : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_rfFreqMax_1                 : std_logic_vector(31 downto 16) := "0000000000010001";--Conversion factor changes according to freqWindows register : Value : X"00000011"
	constant C_PSM_ctuAsVtu_rfFreqMax_0                 : std_logic_vector(15 downto 0) := "1010100010011101";--Conversion factor changes according to freqWindows register : Value : X"0000A89D"
	constant C_PSM_ctuAsVtu_rfFreqMin_1                 : std_logic_vector(31 downto 16) := "0000000000001111";--Conversion factor changes according to freqWindows register : Value : X"0000000F"
	constant C_PSM_ctuAsVtu_rfFreqMin_0                 : std_logic_vector(15 downto 0) := "1111101000010100";--Conversion factor changes according to freqWindows register : Value : X"0000FA14"
	constant C_PSM_ctuAsVtu_rfDiv1Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_rfDiv1Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_rfDiv2Freq_1                : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_rfDiv2Freq_0                : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_syncFreq_1                  : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_syncFreq_0                  : std_logic_vector(15 downto 0) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_syncFreqMax_1               : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_syncFreqMax_0               : std_logic_vector(15 downto 0) := "0000000001101001";--Conversion factor changes according to freqWindows register : Value : X"00000069"
	constant C_PSM_ctuAsVtu_syncFreqMin_1               : std_logic_vector(31 downto 16) := "0000000000000000";--Conversion factor changes according to freqWindows register : Value : X"00000000"
	constant C_PSM_ctuAsVtu_syncFreqMin_0               : std_logic_vector(15 downto 0) := "0000000001011111";--Conversion factor changes according to freqWindows register : Value : X"0000005F"
	constant C_PSM_ctuAsVtu_pdcSyncDelay                : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_pdcOutDelay                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_refPhase                    : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_rfPowerMax                  : std_logic_vector(15 downto 0) := "0111010001111010";-- : Value : X"747A"
	constant C_PSM_ctuAsVtu_rfPowerMin                  : std_logic_vector(15 downto 0) := "0111000111101011";-- : Value : X"71EB"
	constant C_PSM_ctuAsVtu_iodelayControl              : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_PSM_ctuAsVtu_iodelayStatus               : std_logic_vector(15 downto 0) := "1111111111111111";-- : Value : X"FFFF"
	constant C_PSM_ctuAsVtu_ispPowrVersion_1            : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_ispPowrVersion_0            : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_triggerSelector             : std_logic_vector(15 downto 0) := "0000000011001011";-- : Value : X"00CB"
	constant C_PSM_ctuAsVtu_control2                    : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_PSM_ctuAsVtu_xadcStatus                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_xadcAlarms                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_timingControl               : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_PSM_ctuAsVtu_timingStatus                : std_logic_vector(15 downto 0) := "1111111111111111";-- : Value : X"FFFF"
	constant C_PSM_ctuAsVtu_interlock11ASel             : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_interlock11BSel             : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_playMemControl_1            : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_playMemControl_0            : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_sysControl                  : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_PSM_ctuAsVtu_hardwareVersion             : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_designerID                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_boardTemp                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_reserved1                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_testControl                 : std_logic_vector(7 downto 0) := "00000000";-- : Value : X"00"
	constant C_PSM_ctuAsVtu_testStatus                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_firmwareVersion_1           : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_firmwareVersion_0           : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_serialNumber_3              : std_logic_vector(63 downto 48) := "0000000000000000";-- : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_serialNumber_2              : std_logic_vector(47 downto 32) := "0000000000000000";-- : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_serialNumber_1              : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"000000000000"
	constant C_PSM_ctuAsVtu_serialNumber_0              : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_memMapVersion_1             : std_logic_vector(31 downto 16) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_memMapVersion_0             : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"00000000"
	constant C_PSM_ctuAsVtu_rfPowerGain                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_rfPowerOffs                 : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_syncIDelay                  : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_outODelay                   : std_logic_vector(15 downto 0) := "0000000000000000";-- : Value : X"0000"
	constant C_PSM_ctuAsVtu_bValueOffline_3             : std_logic_vector(63 downto 48) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_bValueOffline_2             : std_logic_vector(47 downto 32) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_bValueOffline_1             : std_logic_vector(31 downto 16) := "0000000000000000";--Used in all modes : Value : X"000000000000"
	constant C_PSM_ctuAsVtu_bValueOffline_0             : std_logic_vector(15 downto 0) := "0000000000000000";--Used in all modes : Value : X"00000000"
	constant C_PSM_ctuAsVtu_htValueOffline_3            : std_logic_vector(63 downto 48) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_htValueOffline_2            : std_logic_vector(47 downto 32) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_htValueOffline_1            : std_logic_vector(31 downto 16) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"000000000000"
	constant C_PSM_ctuAsVtu_htValueOffline_0            : std_logic_vector(15 downto 0) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"00000000"
	constant C_PSM_ctuAsVtu_wValueOffline_3             : std_logic_vector(63 downto 48) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_wValueOffline_2             : std_logic_vector(47 downto 32) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_wValueOffline_1             : std_logic_vector(31 downto 16) := "0000000000000000";--Used in windowedOperation mode : Value : X"000000000000"
	constant C_PSM_ctuAsVtu_wValueOffline_0             : std_logic_vector(15 downto 0) := "0000000000000000";--Used in windowedOperation mode : Value : X"00000000"
	constant C_PSM_ctuAsVtu_bValueOnline_3              : std_logic_vector(63 downto 48) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_bValueOnline_2              : std_logic_vector(47 downto 32) := "0000000000000000";--Used in all modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_bValueOnline_1              : std_logic_vector(31 downto 16) := "0000000000000000";--Used in all modes : Value : X"000000000000"
	constant C_PSM_ctuAsVtu_bValueOnline_0              : std_logic_vector(15 downto 0) := "0000000000000000";--Used in all modes : Value : X"00000000"
	constant C_PSM_ctuAsVtu_htValueOnline_3             : std_logic_vector(63 downto 48) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_htValueOnline_2             : std_logic_vector(47 downto 32) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_htValueOnline_1             : std_logic_vector(31 downto 16) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"000000000000"
	constant C_PSM_ctuAsVtu_htValueOnline_0             : std_logic_vector(15 downto 0) := "0000000000000000";--Used in singlePulse, infiniteWindow and windowedOperation modes : Value : X"00000000"
	constant C_PSM_ctuAsVtu_wValueOnline_3              : std_logic_vector(63 downto 48) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_wValueOnline_2              : std_logic_vector(47 downto 32) := "0000000000000000";--Used in windowedOperation mode : Value : X"0000000000000000"
	constant C_PSM_ctuAsVtu_wValueOnline_1              : std_logic_vector(31 downto 16) := "0000000000000000";--Used in windowedOperation mode : Value : X"000000000000"
	constant C_PSM_ctuAsVtu_wValueOnline_0              : std_logic_vector(15 downto 0) := "0000000000000000";--Used in windowedOperation mode : Value : X"00000000"
	-- CODE FIELDS
	-- register-data testControl's code fields :
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTA1 : std_logic_vector(3 downto 0) := "0000";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTA2 : std_logic_vector(3 downto 0) := "0001";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTA3 : std_logic_vector(3 downto 0) := "0010";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTB1 : std_logic_vector(3 downto 0) := "0100";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTB2 : std_logic_vector(3 downto 0) := "0101";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTB3 : std_logic_vector(3 downto 0) := "0110";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTB4 : std_logic_vector(3 downto 0) := "0111";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTC1 : std_logic_vector(3 downto 0) := "1000";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTC2 : std_logic_vector(3 downto 0) := "1001";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_bpTC3 : std_logic_vector(3 downto 0) := "1010";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_fpTiming1 : std_logic_vector(3 downto 0) := "1011";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_fpTiming2 : std_logic_vector(3 downto 0) := "1100";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_fpTiming3 : std_logic_vector(3 downto 0) := "1101";
	constant C_Code_ctuAsVtu_testControl_triggerTestMux_disable : std_logic_vector(3 downto 0) := "1111";
	-- register-data freqWindows's code fields :
	constant C_Code_ctuAsVtu_freqWindows_bpECL4Window_w500us : std_logic_vector(7 downto 7) := "0";
	constant C_Code_ctuAsVtu_freqWindows_bpECL4Window_w50ms : std_logic_vector(7 downto 7) := "1";
	constant C_Code_ctuAsVtu_freqWindows_bpECL3Window_w500us : std_logic_vector(6 downto 6) := "0";
	constant C_Code_ctuAsVtu_freqWindows_bpECL3Window_w50ms : std_logic_vector(6 downto 6) := "1";
	constant C_Code_ctuAsVtu_freqWindows_bpECL2Window_w500us : std_logic_vector(5 downto 5) := "0";
	constant C_Code_ctuAsVtu_freqWindows_bpECL2Window_w50ms : std_logic_vector(5 downto 5) := "1";
	constant C_Code_ctuAsVtu_freqWindows_bpECL1Window_w500us : std_logic_vector(4 downto 4) := "0";
	constant C_Code_ctuAsVtu_freqWindows_bpECL1Window_w50ms : std_logic_vector(4 downto 4) := "1";
	constant C_Code_ctuAsVtu_freqWindows_syncWindow_w500us : std_logic_vector(3 downto 2) := "00";
	constant C_Code_ctuAsVtu_freqWindows_syncWindow_w50ms : std_logic_vector(3 downto 2) := "01";
	constant C_Code_ctuAsVtu_freqWindows_syncWindow_w5s : std_logic_vector(3 downto 2) := "10";
	constant C_Code_ctuAsVtu_freqWindows_syncWindow_w20s : std_logic_vector(3 downto 2) := "11";
	constant C_Code_ctuAsVtu_freqWindows_rfWindow_w500us : std_logic_vector(1 downto 0) := "00";
	constant C_Code_ctuAsVtu_freqWindows_rfWindow_w50ms : std_logic_vector(1 downto 0) := "01";
	constant C_Code_ctuAsVtu_freqWindows_rfWindow_w5s   : std_logic_vector(1 downto 0) := "10";
	constant C_Code_ctuAsVtu_freqWindows_rfWindow_w20s  : std_logic_vector(1 downto 0) := "11";
	-- register-data triggerSelector's code fields :
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTA1 : std_logic_vector(7 downto 4) := "0000";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTA2 : std_logic_vector(7 downto 4) := "0001";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTA3 : std_logic_vector(7 downto 4) := "0010";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTB1 : std_logic_vector(7 downto 4) := "0100";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTB2 : std_logic_vector(7 downto 4) := "0101";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTB3 : std_logic_vector(7 downto 4) := "0110";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTB4 : std_logic_vector(7 downto 4) := "0111";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTC1 : std_logic_vector(7 downto 4) := "1000";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTC2 : std_logic_vector(7 downto 4) := "1001";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_bpTC3 : std_logic_vector(7 downto 4) := "1010";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_fpTiming1 : std_logic_vector(7 downto 4) := "1011";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_fpTiming2 : std_logic_vector(7 downto 4) := "1100";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_fpTiming3 : std_logic_vector(7 downto 4) := "1101";
	constant C_Code_ctuAsVtu_triggerSelector_stopTrigSel_softTrigger : std_logic_vector(7 downto 4) := "1110";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTA1 : std_logic_vector(3 downto 0) := "0000";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTA2 : std_logic_vector(3 downto 0) := "0001";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTA3 : std_logic_vector(3 downto 0) := "0010";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTB1 : std_logic_vector(3 downto 0) := "0100";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTB2 : std_logic_vector(3 downto 0) := "0101";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTB3 : std_logic_vector(3 downto 0) := "0110";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTB4 : std_logic_vector(3 downto 0) := "0111";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTC1 : std_logic_vector(3 downto 0) := "1000";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTC2 : std_logic_vector(3 downto 0) := "1001";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_bpTC3 : std_logic_vector(3 downto 0) := "1010";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_fpTiming1 : std_logic_vector(3 downto 0) := "1011";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_fpTiming2 : std_logic_vector(3 downto 0) := "1100";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_fpTiming3 : std_logic_vector(3 downto 0) := "1101";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_softTrigger : std_logic_vector(3 downto 0) := "1110";
	constant C_Code_ctuAsVtu_triggerSelector_startTrigSel_syncPulse : std_logic_vector(3 downto 0) := "1111";
	-- register-data inoutControl's code fields :
	constant C_Code_ctuAsVtu_inoutControl_bpSyncSel_backplane : std_logic_vector(15 downto 15) := "0";
	constant C_Code_ctuAsVtu_inoutControl_bpSyncSel_frontpanel : std_logic_vector(15 downto 15) := "1";
	constant C_Code_ctuAsVtu_inoutControl_rfDivider2Ratio_divideBy2 : std_logic_vector(14 downto 14) := "0";
	constant C_Code_ctuAsVtu_inoutControl_rfDivider2Ratio_divideBy4 : std_logic_vector(14 downto 14) := "1";
	constant C_Code_ctuAsVtu_inoutControl_syncOperation_syncLessOperation : std_logic_vector(13 downto 13) := "1";
	constant C_Code_ctuAsVtu_inoutControl_syncOperation_syncOperation : std_logic_vector(13 downto 13) := "0";
	constant C_Code_ctuAsVtu_inoutControl_outputStartSel_data : std_logic_vector(12 downto 12) := "0";
	constant C_Code_ctuAsVtu_inoutControl_outputStartSel_pulse : std_logic_vector(12 downto 12) := "1";
	constant C_Code_ctuAsVtu_inoutControl_outputStopSel_data : std_logic_vector(11 downto 11) := "0";
	constant C_Code_ctuAsVtu_inoutControl_outputStopSel_pulse : std_logic_vector(11 downto 11) := "1";
	constant C_Code_ctuAsVtu_inoutControl_rfDivider1Ratio_divideBy4 : std_logic_vector(10 downto 9) := "00";
	constant C_Code_ctuAsVtu_inoutControl_rfDivider1Ratio_divideBy5 : std_logic_vector(10 downto 9) := "10";
	constant C_Code_ctuAsVtu_inoutControl_rfDivider1Ratio_divideBy6 : std_logic_vector(10 downto 9) := "01";
	-- register-data control2's code fields :
	constant C_Code_ctuAsVtu_control2_mode_singlePulse  : std_logic_vector(7 downto 5) := "000";
	constant C_Code_ctuAsVtu_control2_mode_infiniteWindow : std_logic_vector(7 downto 5) := "001";
	constant C_Code_ctuAsVtu_control2_mode_windowedOperation : std_logic_vector(7 downto 5) := "010";
	constant C_Code_ctuAsVtu_control2_mode_syncLessOperation : std_logic_vector(7 downto 5) := "011";
	constant C_Code_ctuAsVtu_control2_mode_lowFreqGeneration : std_logic_vector(7 downto 5) := "100";
	constant C_Code_ctuAsVtu_control2_mode_playMemory   : std_logic_vector(7 downto 5) := "101";
	-- Memory Data
	constant C_Mem_ctuAsVtu_mem2play_Sta     : std_logic_vector(19 downto 1) := "0000100000000000000";--32kb data = 262144 bits (=655.36 us @ 400 MHz) : Word address : "000" & X"4000" ; Byte address : 0x8000
	constant C_Mem_ctuAsVtu_mem2play_End     : std_logic_vector(19 downto 1) := "0000111111111111111";--32kb data = 262144 bits (=655.36 us @ 400 MHz) : Word address : "000" & X"7FFF" ; Byte address : 0xfffe
	-- SubMaps
	constant C_Submap_ctuAsVtu_xadc             : std_logic_vector(19 downto 8) := "000000000011";-- : Word Address : X"300"; Byte Address : 0x300

end;
-- EOF

----------------------------------------------------
--
--      VHDL code generated by Visual Elite
--
--  Design Unit:
--  ------------
--      Unit    Name  :  PlayMemLogic
--      Library Name  :  CTUasVTU
--
--      Creation Date :  Tue Mar 07 11:40:50 2017
--      Version       :  2015.01 v4.6.0 build 42. Date: Feb 22 2015. License: 2015.2
--
--  Options Used:
--  -------------
--      Target
--         Language   :  As Is
--         Purpose    :  Synthesis
--         Vendor     :  Generic
--
--      Style
--         Use Procedures                 :  No
--         Code Destination               :  Combined file
--         Attach Packages                :  Yes
--         Generate Entity                :  Yes
--         Attach Directives              :  Yes
--         Structural                     :  No
--         Configuration Specification    :  No
--         library name in
--         Configuration Specification    :  No
--         Configuration Declaration      :  None
--         Preserve spacing for free text :  Yes
--         Declaration alignment          :  No
--
----------------------------------------------------
----------------------------------------------------
--
--  Library Name :  CTUasVTU
--  Unit    Name :  PlayMemLogic
--  Unit    Type :  Text Unit
--
------------------------------------------------------
----------------------------------------------------
--
--  Library Name :  CTUasVTU
--  Unit    Name :  PlayMemLogic
--  Unit    Type :  Text Unit
--
------------------------------------------------------
------------------------------------------
------------------------------------------
-- Date        : Tue Feb 28 17:09:40 2017
--
-- Author      : Diego Barrientos <diego.barrientos@cern.ch>
--
-- Company     : CERN, BE-RF-CS
--
-- Description : Logic to get address and output logic
--               for play memory.
--
------------------------------------------
------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
library ieee;
use ieee.NUMERIC_STD.all;

entity PlayMemLogic is
    port(
        Clk : in std_logic;
        RunPlayMem : in std_logic;
        FirstOutput : in std_logic;
        PlayingMem : in std_logic;
        LastElem : in std_logic_vector; -- unconstrained
        RdData : in std_logic_vector(7 downto 0);
        RdDataPrev : in std_logic_vector(7 downto 0);
        RdDataZero : in std_logic_vector(7 downto 0);
        FirstBit : in std_logic_vector(2 downto 0);
        LastBit : in std_logic_vector(2 downto 0);
        Mem_Addr : out std_logic_vector; -- unconstrained
        DataOut : out std_logic_vector(7 downto 0)
    );
end PlayMemLogic;

------------------------------------------
------------------------------------------
-- Date        : Tue Feb 28 17:13:42 2017
--
-- Author      : Diego Barrientos <diego.barrientos@cern.ch>
--
-- Company     : CERN, BE-RF-CS
--
-- Description :
--
------------------------------------------
------------------------------------------

architecture V1 of PlayMemLogic is
    -- Constants
    constant CAddrZero : std_logic_vector(Mem_Addr'range) := (others  => '0');
    constant CAddrOne  : std_logic_vector(Mem_Addr'range) := std_logic_vector(to_unsigned(1, Mem_Addr'length));
    constant CAddrTwo  : std_logic_vector(Mem_Addr'range) := std_logic_vector(to_unsigned(2, Mem_Addr'length));

    -- Signals
    signal Mem_Addr_i : std_logic_vector(Mem_Addr'range) := (others => '0'); -- Local copy of Mem_Addr
    signal Mem_Addr_sync : std_logic_vector(Mem_Addr'range) := (others => '0'); -- Auxiliary signal for computing Mem_Addr (synchronous process)
    signal Mem_Addr_sync_prev : std_logic_vector(Mem_Addr'range) := (others => '0'); -- Previous value of Mem_Addr_sync
    signal DataOut_FirstWord : std_logic_vector(7 downto 0) := (others => '0'); -- First word to be put at the output
    signal DataOut_i : std_logic_vector(7 downto 0) := (others => '0'); -- Output word while running with LastElem not zero
    signal DataOut_Zero : std_logic_vector(7 downto 0) := (others => '0'); -- Output word while running with LastElem equal to zero
    signal ZeroInMSBs : std_logic := '0'; -- When active, put RdDataZero in MSBs of output word
    signal FirstBitInt : integer range 0 to 7 := 0; -- Integer conversion of FirstBit
    signal LastBitInt : integer range 0 to 7 := 0; -- Integer conversion of LastBit
    signal FirstSlice  : unsigned(3 downto 0) := (others => '0'); -- Last element of first slice (from 7 to 0) in output word
    signal SecondSlice : unsigned(2 downto 0) := (others => '0'); -- Last element of second slice (from 7 to 0) in output word
    signal LastBitPlayedZero, LastBitPlayedZero_prev : integer range 0 to 7 := 0; -- Last bit of Zero word played in DataOut_Zero
begin

    -- Synchronous process
    process(Clk)
    begin
        if rising_edge(Clk) then
            Mem_Addr_sync_prev <= Mem_Addr_sync;
            if RunPlayMem = '0' then
                -- When not running
                FirstSlice  <= (others => '0');
                SecondSlice <= (others => '0');
                Mem_Addr_sync <= (others => '0');
                ZeroInMSBs <= '0';
                LastBitPlayedZero_prev <= 0;
            elsif LastElem = CAddrZero then
                if PlayingMem = '1' then
                    LastBitPlayedZero_prev <= LastBitPlayedZero;
                else
                    LastBitPlayedZero_prev <= 0;
                end if;
            elsif LastElem = CAddrOne then
                -- Keep Addr always to 1
                Mem_Addr_sync <= CAddrOne;
                -- MSBs always from Zero
                ZeroInMSBs <= '1';
                if FirstOutput = '1' then
                    -- If FirstBitInt < 7: DataOut <= RdDataZero(FirstBitInt to 7) & RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                    -- Else: DataOut <= RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                    FirstSlice <= to_unsigned(FirstBitInt+1, 4);
                    if FirstBitInt >= LastBitInt then
                        SecondSlice <= to_unsigned(FirstBitInt-LastBitInt, 3);
                    else
                        SecondSlice <= (others => '0');
                    end if;
                elsif PlayingMem = '1' then
                    if to_integer(FirstSlice) >= LastBitInt+1 then
                        -- If FirstSlice < 8: DataOut <= RdDataZero(FirstSlice to 7) & RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                        -- Else: DataOut <= RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                        FirstSlice <= '0' & SecondSlice;
                        if to_integer(SecondSlice) > LastBitInt+1 then
                            SecondSlice <= to_unsigned(to_integer(SecondSlice)-LastBitInt-1, 3);
                        else
                            SecondSlice <= (others => '0');
                        end if;
                    else
                        if (to_integer(FirstSlice)-to_integer(SecondSlice)) < LastBitInt+1 then
                            -- DataOut <= RdData(FirstSlice-SecondSlice to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                            FirstSlice <= to_unsigned(8+to_integer(FirstSlice), 4);
                            SecondSlice <= to_unsigned(8+to_integer(FirstSlice)-LastBitInt-1, 3);
                        else
                            -- DataOut <= RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                            FirstSlice <= to_unsigned(8, 4);
                            SecondSlice <= to_unsigned(8-LastBitInt-1, 3);
                        end if;
                    end if;
                end if;
            elsif FirstOutput = '1' then
                -- First output
                -- If FirstBitInt < 7: DataOut <= RdDataPrev(FirstBitInt to 7) & RdData(0 to FirstBitInt-1);
                -- Else: DataOut <= RdData(0 to 7);
                FirstSlice <= to_unsigned(FirstBitInt+1, 4);
                SecondSlice <= (others => '0');
                Mem_Addr_sync <= CAddrTwo;
            elsif PlayingMem = '1' then
                -- While playing memory
                if Mem_Addr_i = LastElem then
                    -- Restart sequence
                    if to_integer(FirstSlice) > LastBitInt+1 then
                        -- Some bits from zero word can be added in next word
                        -- If FirstSlice < 8: DataOut <= RdDataPrev(FirstSlice to 7) & RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                        -- Else: DataOut <= RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                        SecondSlice <= to_unsigned(to_integer(FirstSlice)-LastBitInt-1, 3);
                        Mem_Addr_sync <= CAddrOne;
                    elsif to_integer(FirstSlice) = LastBitInt+1 then
                        -- Prepare jump to Data in next two clock cycles
                        SecondSlice <= (others => '0');
                        Mem_Addr_sync <= CAddrZero;
                    elsif Mem_Addr_sync_prev = Mem_Addr_sync then
                        -- Only execute at second cycle coming to this point
                        if (to_integer(FirstSlice)-to_integer(SecondSlice)) < LastBitInt+1 then
                            -- DataOut <= RdData(FirstSlice-SecondSlice to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                            FirstSlice <= to_unsigned(8+to_integer(FirstSlice), 4);
                            SecondSlice <= to_unsigned(8+to_integer(FirstSlice)-LastBitInt-1, 3);
                        else
                            -- DataOut <= RdData(0 to LastBitInt) & RdDataZero(0 to SecondSlice-1);
                            FirstSlice <= to_unsigned(8, 4);
                            SecondSlice <= to_unsigned(8-LastBitInt-1, 3);
                        end if;
                        Mem_Addr_sync <= CAddrOne;
                    end if;
                    ZeroInMSBs <= '0';
                elsif Mem_Addr_i = CAddrOne then
                    -- Sequence just restarted
                    if SecondSlice /= "000" then
                        -- DataOut <= RdDataZero(FirstSlice to 7) & RdData(0 to FirstSlice-1);
                        FirstSlice <= '0' & SecondSlice;
                        ZeroInMSBs <= '1';
                    else
                        -- Jump to Data
                        -- DataOut <= RdData(0 to 7);
                        FirstSlice <= to_unsigned(8, 4);
                        ZeroInMSBs <= '0';
                    end if;
                    SecondSlice <= (others => '0');
                    Mem_Addr_sync <= CAddrTwo;
                elsif Mem_Addr_i = CAddrZero then
                    -- Sequence restarted with SecondSlice = 0
                    FirstSlice <= (others => '0');
                    Mem_Addr_sync <= CAddrOne;
                    ZeroInMSBs <= '1';
                else
                    Mem_Addr_sync <= std_logic_vector(unsigned(Mem_Addr_i)+1);
                    ZeroInMSBs <= '0';
                end if;
            end if;
        end if;
    end process;

    -- Integer signals
    FirstBitInt <= to_integer(unsigned(FirstBit));
    LastBitInt <= to_integer(unsigned(LastBit));

    -- Concurrent assignment of DataOut_FirstWord
    process(RdDataZero, FirstBitInt)
    begin
        for i in 7 downto 0 loop
            if i <= FirstBitInt then
                -- Reverse order for RdDataZero (Index 0 goes to FirstBitInt)
                DataOut_FirstWord(i) <= RdDataZero(FirstBitInt-i);
            else
                -- Fill with zeros in MSBs
                DataOut_FirstWord(i) <= '0';
            end if;
        end loop;
    end process;

    -- Concurrent assignment of DataOut_i
    process(RdData, RdDataPrev, RdDataZero, FirstSlice, SecondSlice, ZeroInMSBs)
    begin
        for i in 7 downto 0 loop
            if i >= to_integer(FirstSlice) then
                if ZeroInMSBs = '0' then
                    -- MSBs => previous word
                    DataOut_i(i) <= RdDataPrev(7+to_integer(FirstSlice)-i);
                else
                    -- MSBs => zero word
                    DataOut_i(i) <= RdDataZero(7+to_integer(FirstSlice)-i);
                end if;
            elsif i >= to_integer(SecondSlice) then
                -- Intermediate bits => current word
                DataOut_i(i) <= RdData(to_integer(FirstSlice)-i-1);
            else
                -- LSBs => zero word
                DataOut_i(i) <= RdDataZero(to_integer(SecondSlice)-i-1);
            end if;
        end loop;
    end process;

    -- Concurrent assignment of DataOut_Zero
    process(RdDataZero, FirstBitInt, FirstOutput, LastBitInt, LastBitPlayedZero_prev)
        variable last_bit : integer range 0 to 7;
    begin
        if FirstOutput = '1' then
            -- First word
            last_bit := LastBitInt;
            for i in 7 downto 0 loop
                if i > FirstBitInt then
                    -- Fill with zeros in MSBs
                    DataOut_Zero(i) <= '0';
                elsif last_bit > 0 then
                    DataOut_Zero(i) <= RdDataZero(LastBitInt-last_bit);
                    last_bit := last_bit-1;
                else
                    DataOut_Zero(i) <= RdDataZero(LastBitInt);
                    last_bit := LastBitInt;
                end if;
            end loop;
            LastBitPlayedZero <= LastBitInt-last_bit;
        else
            last_bit := LastBitInt-LastBitPlayedZero_prev;
            for i in 7 downto 0 loop
                if last_bit > 0 then
                    DataOut_Zero(i) <= RdDataZero(LastBitInt-last_bit);
                    last_bit := last_bit-1;
                else
                    DataOut_Zero(i) <= RdDataZero(LastBitInt);
                    last_bit := LastBitInt;
                end if;
            end loop;
            LastBitPlayedZero <= LastBitInt-last_bit;
        end if;
    end process;

    -- Output signals
    Mem_Addr_i <= CAddrOne when FirstOutput='1' and LastElem /= CAddrZero else
                  Mem_Addr_sync when PlayingMem='1' and LastElem /= CAddrZero else
                  (others => '0');
    Mem_Addr <= Mem_Addr_i;
    DataOut <= DataOut_FirstWord when FirstOutput='1' and LastElem /= CAddrZero else
               DataOut_i when PlayingMem='1' and LastElem /= CAddrZero else
               DataOut_Zero when PlayingMem='1' and LastElem = CAddrZero else
               (others => '0');

end;

----------------------------------------------------
--
--  Library Name :  CommonVisual
--  Unit    Name :  RSFF
--  Unit    Type :  Text Unit
--
------------------------------------------------------
----------------------------------------------------------------
----------------------------------------------------------------
-- Date        : Wed Jun 11 17:46:39 2003
--
-- Author      : J.C.Molendijk
--
-- Company     : CERN BE-RF-CS
--
-- Description : A Synchronous Set Reset FF (Priority to Reset)
--
----------------------------------------------------------------
-- 20091030 : Addition of the pin reset with
--            default value '0'.
--            Replacement of Clk'event and Clk='1'
--            by rising_edge(Clk).(Done by Greg)
--
----------------------------------------------------------------
----------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity RSFF is
port (Clk,Set,Clr: in std_logic;
      Rst : in std_logic:='0'; --Reset active high, port can be left open is unused
      Q: out std_logic);

end;

----------------------------------------------------------------
----------------------------------------------------------------
-- Date        : Wed Jun 11 17:46:39 2003
--
-- Author      : J.C.Molendijk
--
-- Company     : CERN BE-RF-CS
--
-- Description : A Synchronous Set Reset FF (Priority to Reset)
--
----------------------------------------------------------------
-- 20091030 : Addition of the pin reset with
--            default value '0'.
--            Replacement of Clk'event and Clk='1'
--            by rising_edge(Clk).(Done by Greg)
--
----------------------------------------------------------------
----------------------------------------------------------------
architecture  V1 of RSFF is
begin

process(Clk)
  begin
    if rising_edge(Clk) then
        if Clr = '1' or Rst='1' then
          Q <= '0';
        elsif Set = '1'then
          Q <= '1';
        end if;
    end if;
  end process;

end;


library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.NUMERIC_STD.all;
library work;
use work.ctu_pckg.all;
entity vtuDataShifter is
  generic (
           N : INTEGER := 16;
           --  Number of bits of delay register. Minimum value: 4
           g_DisableDoubleSync : std_logic := '1'
           --  Avoid pulses between two words
           );
  port (
        CoarseZero : out std_logic;
        DataOut : out std_logic_vector(7 downto 0 );
        OutputEnabled : out std_logic;
        Clk : in std_logic;
        Delay : in std_logic_vector(N - 1 downto 0 );
        DataIn : in std_logic_vector(7 downto 0 );
        Enabled : in std_logic;
        SyncPulse : out std_logic
        );


end vtuDataShifter;


architecture vtuDataShifter of vtuDataShifter is

  signal SyncPulse_i : std_logic;
  signal DataClean : std_logic_vector(7 downto 0 );
  signal DataOut_i : std_logic_vector(7 downto 0 );
  signal CoarseCnt : std_logic_vector(N - 4 downto 0 );
  signal ClearOutGoodNext : std_logic;
  signal CZeroDly : std_logic_vector(N - 4 downto 0 );
  signal C_Zero8 : std_logic_vector(7 downto 0 );
  signal Ready : std_logic;
  signal CoarseGood : std_logic;
  signal OutGoodNext : std_logic;
  signal DataIn_clean : std_logic_vector(7 downto 0 );
  signal OutMuxSel : std_logic;
  signal Sync_prev : std_logic;
  signal CoarseDly : std_logic_vector(N - 4 downto 0 );
  signal OutputEnabled_i : std_logic;
  signal CntRst : std_logic;
  signal CoarseDly_i : std_logic_vector(N - 4 downto 0 );
  signal Sync_i : std_logic;
  signal OutGood : std_logic;
  signal COne : std_logic_vector(N - 4 downto 0 );
  signal Disabled : std_logic;
  signal DataRaw : std_logic_vector(15 downto 0 );
  signal CoarseZero_i : std_logic;
  signal RemoveDoubleSync : std_logic;
  signal UseNextValue : std_logic;
  signal FineDly : std_logic_vector(2 downto 0 );
  component RSFF
      port (
            Clk : in std_logic;
            Set : in std_logic;
            Clr : in std_logic;
            Rst : in std_logic := '0';
            Q : out std_logic
            );
  end component;
  signal visual_B_Counter_cur_state : std_logic_vector((N - 4) - (0) + 1 - 1
                                       downto 0 );
  signal visual_B_Counter_next_state : std_logic_vector((N - 4) - (0) + 1 - 1
                                        downto 0 );
  signal visual_B_Counter_next_count : std_logic_vector((N - 4) - (0) + 1 - 1
                                        downto 0 );
  signal visual_B_CoarseDly_sub_dif_int : std_logic_vector((N - 4) - (0) + 1
                                           downto 0 );
  signal visual_B_CoarseDly_sub_tmp_a : std_logic_vector((N - 4) - (0) + 1 - 1
                                         downto 0 );
  signal visual_B_CoarseDly_sub_tmp_b : std_logic_vector((N - 4) - (0) + 1 - 1
                                         downto 0 );
  constant visual_B_CoarseDly_sub_zero : std_logic_vector((N - 4) - (0) + 1 - 1
                                          downto 0 ) := (others  => '0');

begin

  B_ReadyFF: RSFF
    port map (
              Clk => Clk,
              Set => SyncPulse_i,
              Clr => Disabled,
              Rst => open,
              Q => Ready
              );

  B_OutGoodNext: RSFF
    port map (
              Clk => Clk,
              Set => OutGood,
              Clr => ClearOutGoodNext,
              Rst => open,
              Q => OutGoodNext
              );

  DataIn_clean <= clean_data(DataIn);

   Sync_i <= ( DataIn_clean(7)) or ( DataIn_clean(6)) or ( DataIn_clean(5))
             or ( DataIn_clean(4)) or ( DataIn_clean(3)) or ( DataIn_clean(2))
             or ( DataIn_clean(1)) or ( DataIn_clean(0));

  CoarseCnt(N-4 downto 0) <= (visual_B_Counter_cur_state);


  visual_B_Counter_next_state <= visual_B_Counter_next_count;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then
    if (CntRst = '1') then
      visual_B_Counter_cur_state <= (others => '0');
   else
       visual_B_Counter_cur_state <= visual_B_Counter_next_state;
   end if;
  end if;
  end process;

  process (visual_B_Counter_cur_state )

  begin

    visual_B_Counter_next_count <= std_logic_vector(unsigned
  (visual_B_Counter_cur_state) + 1);

  end process;

  CoarseDly_i(N-4 downto 0) <= Delay(N-1 downto 3);

  FineDly(2 downto 0) <= Delay(2 downto 0);

   CntRst <= ( SyncPulse_i) or ( Disabled);

  process (C_Zero8 , DataOut_i , OutputEnabled_i)
   begin
     case OutputEnabled_i is
       when '0' =>
         DataOut(7 downto 0) <=  C_Zero8(7 downto 0);
       when others =>
         DataOut(7 downto 0) <=  DataOut_i(7 downto 0);
     end case;
   end process;

  process (CoarseCnt , CoarseDly)
   begin
      if ((CoarseCnt(N-4 downto 0)) = (CoarseDly(N-4 downto 0))) then
        CoarseGood <= '1';
      else
        CoarseGood <= '0';
      end if;
  end process;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then

      if (SyncPulse_i = '1') then
        DataClean(7 downto 0) <= (DataIn_clean(7 downto 0));
      end if;

  end if;
  end process;

  process (OutGood , OutGoodNext , UseNextValue)
   begin
     case UseNextValue is
       when '0' =>
         OutMuxSel <=  OutGood;
       when others =>
         OutMuxSel <=  OutGoodNext;
     end case;
   end process;

  ShiftReg_proc: process(DataRaw, FineDly)
    variable DataOut_aux : std_logic_vector(15 downto 0);
  begin
    DataOut_aux(15 downto 0) := std_logic_vector(shift_right(unsigned(DataRaw),
   to_integer(unsigned(FineDly))));
    UseNextValue <= or_reduce(DataOut_aux(7 downto 0));
    if or_reduce(DataOut_aux(7 downto 0))='0' then
      DataOut_i(7 downto 0) <= DataOut_aux(15 downto 8);
    else
      DataOut_i(7 downto 0) <= DataOut_aux(7 downto 0);
    end if;
  end process;

  DataRaw(15 downto 8) <= DataClean(7 downto 0);
  DataRaw(7 downto 0) <= C_Zero8(7 downto 0);

      C_Zero8(7 downto 0) <= (others => '0');

  SyncPulse <= SyncPulse_i;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then
        Sync_prev <= (Sync_i);


  end if;
  end process;

   OutGood <= ( CoarseGood) and ( Ready);

  process (CoarseDly_i , CZeroDly)
   begin
      if ((CoarseDly_i(N-4 downto 0)) = (CZeroDly(N-4 downto 0))) then
        CoarseZero_i <= '1';
      else
        CoarseZero_i <= '0';
      end if;
  end process;

      CZeroDly(N-4 downto 0) <= (others => '0');

   Disabled <= (not Enabled) or ( CoarseZero_i);

    visual_B_CoarseDly_sub_tmp_a <= (CoarseDly_i(N-4 downto 0));
    visual_B_CoarseDly_sub_tmp_b <= (COne(N-4 downto 0));
    visual_B_CoarseDly_sub_dif_int <= std_logic_vector((unsigned('0' &
   visual_B_CoarseDly_sub_tmp_a)
       - unsigned('0' & visual_B_CoarseDly_sub_tmp_b)));

       CoarseDly(N-4 downto 0) <= (visual_B_CoarseDly_sub_dif_int((N-4) - (0) +
   1 - 1 downto 0));

  COne(N-4 downto 0) <= (0=>'1', others=>'0');

   SyncPulse_i <= ((not RemoveDoubleSync) or (not Sync_prev)) and (( Sync_i));

   ClearOutGoodNext <= (not UseNextValue) or ( Disabled) or ( OutGoodNext);

   OutputEnabled_i <= ( OutMuxSel) and (not Disabled);

  CoarseZero <= CoarseZero_i;

  RemoveDoubleSync <= g_DisableDoubleSync;

  OutputEnabled <= OutputEnabled_i;
end vtuDataShifter;

library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.NUMERIC_STD.all;
entity vtuSeq is
  port (
        Clk : in std_logic;
        Rst : in std_logic;
        Start : in std_logic;
        Stop : in std_logic;
        SyncPulse : in std_logic;
        OutputEnable1 : in std_logic;
        OutputEnable2 : in std_logic;
        Shifter1Ena : out std_logic := '0';
        Shifter2Ena : out std_logic := '0';
        SwitchtoHT : out std_logic := '0';
        wValueOne : in std_logic;
        WindowDone : in std_logic;
        RestartEnable : in std_logic;
        CounterRst : out std_logic := '0';
        Run : out std_logic := '0'
        );

end vtuSeq;


architecture vtuSeq of vtuSeq is

  type visual_Idle_states is (Idle, S_BValue, S_HTValue, S_waitSync);
  signal visual_Idle_current : visual_Idle_states;


begin



  -- Synchronous process
  vtuSeq_Idle:
  process (Clk)
  begin

    if (rising_edge(Clk)) then
      if (Rst = '1' or Stop = '1') then
        Shifter1Ena <= '1';
        Shifter2Ena <= '0';
        SwitchtoHT   <= '0';
        CounterRst <= '1';
        Run <= '0';
        visual_Idle_current <= Idle;
      else

        case visual_Idle_current is
          when Idle =>
            if (Start = '1' and SyncPulse = '0') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '0';
              SwitchtoHT   <= '0';
              CounterRst <= '0';
              Run <= '0';
              visual_Idle_current <= S_waitSync;
            elsif (Start = '1' and SyncPulse = '1') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '1';
              SwitchtoHT   <= '0';
              CounterRst <= '0';
              Run <= '1';
              visual_Idle_current <= S_BValue;
            else
              Shifter1Ena <= '1';
              Shifter2Ena <= '0';
              SwitchtoHT   <= '0';
              CounterRst <= '0';
              Run <= '0';
              visual_Idle_current <= Idle;
            end if;

          when S_BValue =>
            if (OutputEnable1 = '1' and wValueOne = '0') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '1';
              SwitchtoHT   <= '1';
              CounterRst <= '0';
              visual_Idle_current <= S_HTValue;
            elsif (OutputEnable1 = '1' and wValueOne = '1') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '0';
              SwitchtoHT   <= '0';
              CounterRst <= '1';
              Run <= '0';
              visual_Idle_current <= Idle;
            elsif (Start = '1' and SyncPulse = '0') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '0';
              SwitchtoHT   <= '0';
              CounterRst <= '1';
              Run <= '0';
              visual_Idle_current <= S_waitSync;
            elsif (Start = '1' and SyncPulse = '1') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '1';
              SwitchtoHT   <= '0';
              CounterRst <= '1';
              visual_Idle_current <= S_BValue;
            else
              CounterRst <= '0';
              visual_Idle_current <= S_BValue;
            end if;

          when S_HTValue =>
            if (OutputEnable2 = '1' and WindowDone = '1') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '0';
              SwitchtoHT   <= '0';
              CounterRst <= '1';
              Run <= '0';
              visual_Idle_current <= Idle;
            elsif (RestartEnable = '1' and Start = '1' and SyncPulse = '0') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '0';
              SwitchtoHT   <= '0';
              CounterRst <= '1';
              Run <= '0';
              visual_Idle_current <= S_waitSync;
            elsif (RestartEnable = '1' and Start = '1' and SyncPulse = '1') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '1';
              SwitchtoHT   <= '0';
              CounterRst <= '1';
              visual_Idle_current <= S_BValue;
            else
              visual_Idle_current <= S_HTValue;
            end if;

          when S_waitSync =>
            if (SyncPulse = '1') then
              Shifter1Ena <= '1';
              Shifter2Ena <= '1';
              SwitchtoHT   <= '0';
              CounterRst <= '0';
              Run <= '1';
              visual_Idle_current <= S_BValue;
            else
              CounterRst <= '0';
              visual_Idle_current <= S_waitSync;
            end if;

          when others =>

            visual_Idle_current <= Idle;
        end case;
      end if;
    end if;
  end process vtuSeq_Idle;

end vtuSeq;
----------------------------------------------------
--
--  Library Name :  CommonVisual
--  Unit    Name :  LimSubtrN
--  Unit    Type :  Text Unit
--
------------------------------------------------------
--------------------------------------------------------------------
--------------------------------------------------------------------
-- Date        : Thu Oct 22 15:44:29 2009
--
-- Author      : Gregoire Hagmann
--
-- Company     : CERN BE/RF/FB
--
-- Description : Subtractor with signed data bus and limiter circuitry
--
--------------------------------------------------------------------
--------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
library ieee;
use ieee.NUMERIC_STD.all;


entity  LimSubtrN  is
port (A : in std_logic_vector; --vector size defined by the signals connected to the block
      B : in std_logic_vector ;
      lim : out std_logic;
      O : out std_logic_vector);

end;

--------------------------------------------------------------------
--------------------------------------------------------------------
-- Date        : Thu Oct 22 15:44:29 2009
--
-- Author      : Gregoire Hagmann
--
-- Company     : CERN BE/RF/FB
--
-- Description : Subtractor with signed data bus and limiter circuitry
--
--------------------------------------------------------------------
--------------------------------------------------------------------
architecture  V1  of  LimSubtrN  is

    signal l_sum : std_logic_vector(O'length downto 0):=(others => '0'); --default value for simulation only
    constant c_ValMax : std_logic_vector(O'range):=('0',others => '1');
    constant c_ValMin : std_logic_vector(O'range):=('1',others => '0');
begin
    process(l_sum)
    begin
        if l_sum(O'length)='0' and l_sum(O'length-1)='1' then
            --Val max positive
            O <= c_ValMax;
            Lim <= '1';
        elsif l_sum(O'length)='1' and l_sum(O'length-1)='0' then
            --Val max negative
            O <= c_ValMin;
            Lim <= '1';
        else
            O <= l_sum(O'length-1 downto 0);
            Lim <= '0';
        end if;
    end process;

    l_sum <= std_logic_vector(resize(signed(A),O'length+1) - resize(signed(B),O'length+1));


end;

----------------------------------------------------
--
--  Library Name :  CommonVisual
--  Unit    Name :  LimAdderN
--  Unit    Type :  Text Unit
--
------------------------------------------------------
--------------------------------------------------------------------
--------------------------------------------------------------------
-- Date        : Thu Oct 22 15:44:29 2009
--
-- Author      : Gregoire Hagmann
--
-- Company     : CERN BE/RF/FB
--
-- Description : Adder with signed data bus and limiter circuitry
--
--------------------------------------------------------------------
--------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
library ieee;
use ieee.NUMERIC_STD.all;


entity  LimAdderN  is
port (A : in std_logic_vector; --vector size defined by the signals connected to the block
      B : in std_logic_vector ;
      lim : out std_logic;
      O : out std_logic_vector);

end;

--------------------------------------------------------------------
--------------------------------------------------------------------
-- Date        : Thu Oct 22 15:44:29 2009
--
-- Author      : Gregoire Hagmann
--
-- Company     : CERN BE/RF/FB
--
-- Description : Adder with signed data bus and limiter circuitry
--
--------------------------------------------------------------------
--------------------------------------------------------------------
architecture  V1  of  LimAdderN  is

    signal l_sum : std_logic_vector(O'length downto 0):=(others => '0'); --default value for simulation only
    constant c_ValMax : std_logic_vector(O'range):=('0',others => '1');
    constant c_ValMin : std_logic_vector(O'range):=('1',others => '0');
begin
    process(l_sum)
    begin
        if l_sum(O'length)='0' and l_sum(O'length-1)='1' then
            --Val max positive
            O <= c_ValMax;
            Lim <= '1';
        elsif l_sum(O'length)='1' and l_sum(O'length-1)='0' then
            --Val max negative
            O <= c_ValMin;
            Lim <= '1';
        else
            O <= l_sum(O'length-1 downto 0);
            Lim <= '0';
        end if;
    end process;

    l_sum <= std_logic_vector(resize(signed(A),O'length+1) + resize(signed(B),O'length+1));


end;


library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.NUMERIC_STD.all;
library work;
use work.MemMap_ctuAsVtu.all;
entity WrongValuesLogic is
  port (
        wrongB : out std_logic := '0';
        wrongHT : out std_logic := '0';
        wrongW : out std_logic := '0';
        BCoarseZero : in std_logic;
        HTCoarseZero : in std_logic;
        wValueOne : in std_logic;
        wValueZero : in std_logic;
        Mode : in std_logic_vector(2 downto 0 );
        Clk : in std_logic;
        Rst : in std_logic
        );

end WrongValuesLogic;


architecture WrongValuesLogic of WrongValuesLogic is


begin
  WrongValuesLogic:
  process (Clk)
  begin

    if rising_edge(Clk) then
      if Rst = '1' then
        wrongB <= '1';
        wrongHT <= '1';
        wrongW <= '1';
      else
        if (Mode = C_Code_ctuAsVtu_control2_mode_singlePulse) AND
           (BCoarseZero = '1') then
          wrongB <= '1';
          wrongHT <= '0';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_infiniteWindow) AND
              (BCoarseZero = '1') then
          wrongB <= '1';
          wrongHT <= '0';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_infiniteWindow) AND
              (HTCoarseZero = '1') then
          wrongB <= '0';
          wrongHT <= '1';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_windowedOperation) AND
              (BCoarseZero = '1') then
          wrongB <= '1';
          wrongHT <= '0';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_windowedOperation) AND
              (HTCoarseZero = '1') then
          wrongB <= '0';
          wrongHT <= '1';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_windowedOperation) AND
              (wValueOne = '1') then
          wrongB <= '0';
          wrongHT <= '0';
          wrongW <= '1';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_windowedOperation) AND
              (wValueZero = '1') then
          wrongB <= '0';
          wrongHT <= '0';
          wrongW <= '1';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_syncLessOperation) AND
              (HTCoarseZero = '1') then
          wrongB <= '0';
          wrongHT <= '1';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_lowFreqGeneration) AND
              (BCoarseZero = '1') then
          wrongB <= '1';
          wrongHT <= '0';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_lowFreqGeneration) AND
              (HTCoarseZero = '1') then
          wrongB <= '0';
          wrongHT <= '1';
          wrongW <= '0';
        elsif
              (Mode = C_Code_ctuAsVtu_control2_mode_playMemory) AND
              (BCoarseZero = '1') then
          wrongB <= '1';
          wrongHT <= '0';
          wrongW <= '0';
        else
          wrongB <= '0';
          wrongHT <= '0';
          wrongW <= '0';
        end if ;
      end if ;
    end if ;
  end process WrongValuesLogic;

end WrongValuesLogic;

library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.NUMERIC_STD.all;
library work;
use work.MemMap_ctuAsVtu.all;
entity ModeSelDecoder is
  port (
        Mode : in std_logic_vector(2 downto 0 );
        SinglePulseMode : out std_logic;
        InfiniteWindowMode : out std_logic;
        WindowedOperationMode : out std_logic;
        SyncLessOperationMode : out std_logic;
        LowFreqGenerationMode : out std_logic;
        PlayMemoryMode : out std_logic
        );

end ModeSelDecoder;


architecture ModeSelDecoder of ModeSelDecoder is


begin
  ModeSelDecoder:
  process (Mode)
  begin

    if (Mode = C_Code_ctuAsVtu_control2_mode_singlePulse) then
      SinglePulseMode <= '1';
      InfiniteWindowMode <= '0';
      WindowedOperationMode <= '0';
      SyncLessOperationMode <= '0';
      LowFreqGenerationMode <= '0';
      PlayMemoryMode <= '0';
    elsif (Mode = C_Code_ctuAsVtu_control2_mode_infiniteWindow) then
      SinglePulseMode <= '0';
      InfiniteWindowMode <= '1';
      WindowedOperationMode <= '0';
      SyncLessOperationMode <= '0';
      LowFreqGenerationMode <= '0';
      PlayMemoryMode <= '0';
    elsif (Mode = C_Code_ctuAsVtu_control2_mode_windowedOperation) then
      SinglePulseMode <= '0';
      InfiniteWindowMode <= '0';
      WindowedOperationMode <= '1';
      SyncLessOperationMode <= '0';
      LowFreqGenerationMode <= '0';
      PlayMemoryMode <= '0';
    elsif (Mode = C_Code_ctuAsVtu_control2_mode_syncLessOperation) then
      SinglePulseMode <= '0';
      InfiniteWindowMode <= '0';
      WindowedOperationMode <= '0';
      SyncLessOperationMode <= '1';
      LowFreqGenerationMode <= '0';
      PlayMemoryMode <= '0';
    elsif (Mode = C_Code_ctuAsVtu_control2_mode_lowFreqGeneration) then
      SinglePulseMode <= '0';
      InfiniteWindowMode <= '0';
      WindowedOperationMode <= '0';
      SyncLessOperationMode <= '0';
      LowFreqGenerationMode <= '1';
      PlayMemoryMode <= '0';
    elsif (Mode = C_Code_ctuAsVtu_control2_mode_playMemory) then
      SinglePulseMode <= '0';
      InfiniteWindowMode <= '0';
      WindowedOperationMode <= '0';
      SyncLessOperationMode <= '0';
      LowFreqGenerationMode <= '0';
      PlayMemoryMode <= '1';
    else
      SinglePulseMode <= '0';
      InfiniteWindowMode <= '0';
      WindowedOperationMode <= '0';
      SyncLessOperationMode <= '0';
      LowFreqGenerationMode <= '0';
      PlayMemoryMode <= '0';
    end if ;
  end process ModeSelDecoder;

end ModeSelDecoder;

library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.NUMERIC_STD.all;
library work;
use work.MemMap_ctuAsVtu.all;
entity vtuCore is
  port (
    --  Interface to the memory for the play memory mode.
        Mem_RdDone : in std_logic;
        Mem_Addr : out std_logic_vector(14 downto 0 );
        Mem_RdData : in std_logic_vector(7 downto 0 );
        Mem_RdEna : out std_logic;

        --  Mode:
        --   000: single pulse
        --   001: infinite window
        --   010: windowed operation,
        --   011: syncless operation
        --   100: low frequency generation
        --   101: play memory
        Mode : in std_logic_vector(2 downto 0 );

        --  Parameter:
        --   For single pulse mode: number of clock after the sync pulse.
        bValue : in std_logic_vector(63 downto 0 );
        wrongB : out std_logic;

        --  Parameter: first synchronize with sync (?)
        UseSyncAsStart : in std_logic;

        --  Parameter: number of pulses
        htValue : in std_logic_vector(63 downto 0 );
        htSwitching : in std_logic;
        wrongHT : out std_logic;

        --  Parameter: window size
        wValue : in std_logic_vector(63 downto 0 );
        wrongW : out std_logic;

        --  Inputs.
        Start : in std_logic;
        Stop : in std_logic;

        DataIn : in std_logic_vector(7 downto 0 );

        --  Output.
        DataOut : out std_logic_vector(7 downto 0 );

        --  Parameter: position of the last bit to be played.
        Mem_LastBit : in std_logic_vector(2 downto 0 );
        Mem_LastElem : in std_logic_vector(14 downto 0 );

        SyncPulse : out std_logic;
        Rst : in std_logic;--  Synchronous to QCLk
        Run : out std_logic;
        Clk : in std_logic
        );


end vtuCore;


architecture vtuCore of vtuCore is

  signal Run_n : std_logic;
  signal FilledMuxSel : std_logic;
  signal DataOut_seq_i : std_logic_vector(7 downto 0 );
  signal RunSyncLess : std_logic;
  signal RestartEnable : std_logic;
  signal wrongValue : std_logic;
  signal SwitchHTeffective_n : std_logic;
  signal ClkValueSwitch_n : std_logic;
  signal CZero : std_logic_vector(63 downto 0 );
  signal SyncLessEna : std_logic;
  signal DataOutLowFreq : std_logic_vector(7 downto 0 );
  signal DataIn_2 : std_logic_vector(7 downto 0 );
  signal Shifter2Ena : std_logic;
  signal DataInHTSyncLess : std_logic_vector(7 downto 0 );
  signal AllOneOut : std_logic_vector(7 downto 0 );
  signal WindowDone : std_logic;
  signal wValueOne_seq : std_logic;
  signal BCoarseZero : std_logic;
  signal SwitchOutput : std_logic;
  signal COne : std_logic_vector(63 downto 0 );
  signal Run_i : std_logic;
  signal PlayMemoryMode : std_logic;
  signal wrongHT_s : std_logic;
  signal counterEnable : std_logic;
  signal DataOutPlayMem : std_logic_vector(7 downto 0 );
  signal SwitchtoHT : std_logic;
  signal wValue_effective : std_logic_vector(63 downto 0 );
  signal Shifter1Ena : std_logic;
  signal DataOut_2 : std_logic_vector(7 downto 0 );
  signal HTSwitchEna : std_logic;
  signal PulseCount : std_logic_vector(63 downto 0 );
  signal SetStartData : std_logic := '0';
  signal wrongW_s : std_logic;
  signal WindowedOperationMode : std_logic;
  signal DataOutPulse : std_logic;
  signal NoStart : std_logic;
  signal InfiniteWindowMode : std_logic;
  signal DataFilled : std_logic_vector(7 downto 0 );
  signal HTCoarseZero : std_logic;
  signal wValueZero : std_logic;
  signal SwitchHTeffective : std_logic;
  signal WindowDone_seq : std_logic;
  signal htValue_effective : std_logic_vector(63 downto 0 );
  signal wrongB_s : std_logic;
  signal wValueOne : std_logic;
  signal DataAllEqual : std_logic_vector(7 downto 0 );
  signal StartSyncLess : std_logic;
  signal counterReset : std_logic;
  signal SyncLessDisabled : std_logic;
  signal LowFreqGenerationMode : std_logic;
  signal ClkValueSwitch : std_logic;
  signal SinglePulseMode : std_logic;
  signal InfiniteWindow : std_logic;
  signal DataOut_seq : std_logic_vector(7 downto 0 );
  signal DataOutHTSyncLess : std_logic_vector(7 downto 0 );
  signal htValuePlusOne : std_logic_vector(63 downto 0 );
  signal SyncLessOperationMode : std_logic;
  signal AllZeroOut : std_logic_vector(7 downto 0 );
  signal Start_i : std_logic;
  signal Mem_RdDataZero : std_logic_vector(7 downto 0 ) := (others  => '0');
  signal Run_seq : std_logic;
  signal PlayingMem_prev : std_logic;
  signal StopPlayMem : std_logic;
  signal SyncPulse_i : std_logic;
  signal FirstOutput : std_logic;
  signal RstOrStopSeq : std_logic;
  signal FirstBit : std_logic_vector(2 downto 0 );
  signal Mem_Addr_i : std_logic_vector(14 downto 0 );
  signal OE_SyncLess : std_logic;
  signal RunPlayMem : std_logic;
  signal PlayingMem : std_logic;
  signal Stop_seq : std_logic;
  signal SyncLessStartData : std_logic_vector(7 downto 0 );
  signal Start_seq : std_logic;
  signal DataOut_1 : std_logic_vector(7 downto 0 );
  signal SetPlayingMem : std_logic;
  signal PlayingMem_i : std_logic;
  signal MemAddrIsZero : std_logic;
  signal FirstSyncArrived : std_logic := '0';
  signal DisconnectStart : std_logic;
  signal Mem_AddrZero : std_logic_vector(14 downto 0 );
  signal Mem_RdData_prev : std_logic_vector(7 downto 0 ) := (others  => '0');
  signal OE_1 : std_logic;
  signal OE_2 : std_logic;
  signal MemAddrIsZeroPrev : std_logic;
  signal StartPlayMem : std_logic;
  signal RunPlayAndSyncPulse : std_logic;
  signal RunPlayMem_i : std_logic;
  signal ReadDataZero : std_logic;
  component vtuDataShifter
      generic (
               N : INTEGER := 16;
               g_DisableDoubleSync : std_logic := '1'
               );
      port (
            CoarseZero : out std_logic;
            DataOut : out std_logic_vector(7 downto 0 );
            OutputEnabled : out std_logic;
            Clk : in std_logic;
            Delay : in std_logic_vector(N - 1 downto 0 );
            DataIn : in std_logic_vector(7 downto 0 );
            Enabled : in std_logic;
            SyncPulse : out std_logic
            );
  end component;
  component LimSubtrN
      port (
            A : in std_logic_vector;
            B : in std_logic_vector;
            lim : out std_logic;
            O : out std_logic_vector
            );
  end component;
  component vtuSeq
      port (
            Clk : in std_logic;
            Rst : in std_logic;
            Start : in std_logic;
            Stop : in std_logic;
            SyncPulse : in std_logic;
            OutputEnable1 : in std_logic;
            OutputEnable2 : in std_logic;
            Shifter1Ena : out std_logic;
            Shifter2Ena : out std_logic;
            SwitchtoHT : out std_logic;
            wValueOne : in std_logic;
            WindowDone : in std_logic;
            RestartEnable : in std_logic;
            CounterRst : out std_logic;
            Run : out std_logic
            );
  end component;
  component RSFF
      port (
            Clk : in std_logic;
            Set : in std_logic;
            Clr : in std_logic;
            Rst : in std_logic := '0';
            Q : out std_logic
            );
  end component;
  component PlayMemLogic
      port (
            Clk : in std_logic;
            RunPlayMem : in std_logic;
            FirstOutput : in std_logic;
            PlayingMem : in std_logic;
            LastElem : in std_logic_vector;
            RdData : in std_logic_vector(7 downto 0 );
            RdDataPrev : in std_logic_vector(7 downto 0 );
            RdDataZero : in std_logic_vector(7 downto 0 );
            FirstBit : in std_logic_vector(2 downto 0 );
            LastBit : in std_logic_vector(2 downto 0 );
            Mem_Addr : out std_logic_vector;
            DataOut : out std_logic_vector(7 downto 0 )
            );
  end component;
  component ModeSelDecoder
      port (
            Mode : in std_logic_vector(2 downto 0 );
            SinglePulseMode : out std_logic;
            InfiniteWindowMode : out std_logic;
            WindowedOperationMode : out std_logic;
            SyncLessOperationMode : out std_logic;
            LowFreqGenerationMode : out std_logic;
            PlayMemoryMode : out std_logic
            );
  end component;
  component WrongValuesLogic
      port (
            wrongB : out std_logic;
            wrongHT : out std_logic;
            wrongW : out std_logic;
            BCoarseZero : in std_logic;
            HTCoarseZero : in std_logic;
            wValueOne : in std_logic;
            wValueZero : in std_logic;
            Mode : in std_logic_vector(2 downto 0 );
            Clk : in std_logic;
            Rst : in std_logic
            );
  end component;
  component LimAdderN
      port (
            A : in std_logic_vector;
            B : in std_logic_vector;
            lim : out std_logic;
            O : out std_logic_vector
            );
  end component;
  signal visual_B_PulseCounter_cur_state : std_logic_vector(64 - 1 downto 0 );
  signal visual_B_PulseCounter_next_state : std_logic_vector(64 - 1 downto 0 );
  signal visual_B_PulseCounter_next_count : std_logic_vector(64 - 1 downto 0 );
  signal visual_B_PulseCounter_en_state : std_logic_vector(64 - 1 downto 0 );
  signal visual_B_ClkValueSwitchFF_Q : std_logic;
  signal visual_B_SwitchHTFF_Q : std_logic;

begin

  B_DataShifterB: vtuDataShifter
    generic map (N => 64,
                 g_DisableDoubleSync => '1'
                 )
    port map (
              CoarseZero => BCoarseZero,
              DataOut => DataOut_1(7 downto 0),
              OutputEnabled => OE_1,
              Clk => Clk,
              Delay => bValue(63 downto 0),
              DataIn => DataIn(7 downto 0),
              Enabled => Shifter1Ena,
              SyncPulse => SyncPulse_i
              );

  B_DataShifterHT: vtuDataShifter
    generic map (N => 64,
                 g_DisableDoubleSync => '0'
                 )
    port map (
              CoarseZero => HTCoarseZero,
              DataOut => DataOut_2(7 downto 0),
              OutputEnabled => OE_2,
              Clk => Clk,
              Delay => htValue_effective(63 downto 0),
              DataIn => DataIn_2(7 downto 0),
              Enabled => Shifter2Ena,
              SyncPulse => open
              );

  B_LimSubtr: LimSubtrN
    port map (
              A => wValue(63 downto 0),
              B => COne(63 downto 0),
              lim => open,
              O => wValue_effective(63 downto 0)
              );

  B_vtuSeq: vtuSeq
    port map (
              Clk => Clk,
              Rst => Rst,
              Start => Start_seq,
              Stop => Stop_seq,
              SyncPulse => SyncPulse_i,
              OutputEnable1 => OE_1,
              OutputEnable2 => OE_2,
              Shifter1Ena => Shifter1Ena,
              Shifter2Ena => Shifter2Ena,
              SwitchtoHT => SwitchtoHT,
              wValueOne => wValueOne_seq,
              WindowDone => WindowDone_seq,
              RestartEnable => RestartEnable,
              CounterRst => counterReset,
              Run => Run_seq
              );

  B_DataShifterHT_SyncLess: vtuDataShifter
    generic map (N => 64,
                 g_DisableDoubleSync => '0'
                 )
    port map (
              CoarseZero => open,
              DataOut => DataOutHTSyncLess(7 downto 0),
              OutputEnabled => OE_SyncLess,
              Clk => Clk,
              Delay => htValue_effective(63 downto 0),
              DataIn => DataInHTSyncLess(7 downto 0),
              Enabled => SyncLessEna,
              SyncPulse => open
              );

  B_RSFFSyncLessMux: RSFF
    port map (
              Clk => Clk,
              Set => StartSyncLess,
              Clr => SetStartData,
              Rst => Rst,
              Q => SetStartData
              );

  B_RSFFRunSyncLess: RSFF
    port map (
              Clk => Clk,
              Set => StartSyncLess,
              Clr => SyncLessDisabled,
              Rst => Rst,
              Q => RunSyncLess
              );

  B_PlayMemRunningFF: RSFF
    port map (
              Clk => Clk,
              Set => StartPlayMem,
              Clr => StopPlayMem,
              Rst => Rst,
              Q => RunPlayMem_i
              );

  B_FirstSyncFF: RSFF
    port map (
              Clk => Clk,
              Set => RunPlayAndSyncPulse,
              Clr => StopPlayMem,
              Rst => Rst,
              Q => FirstSyncArrived
              );

  B_PlayingMemFF: RSFF
    port map (
              Clk => Clk,
              Set => SetPlayingMem,
              Clr => StopPlayMem,
              Rst => Rst,
              Q => PlayingMem_i
              );

  B_PlayMemLogic: PlayMemLogic
    port map (
              Clk => Clk,
              RunPlayMem => RunPlayMem,
              FirstOutput => FirstOutput,
              PlayingMem => PlayingMem,
              LastElem => Mem_LastElem(14 downto 0),
              RdData => Mem_RdData(7 downto 0),
              RdDataPrev => Mem_RdData_prev(7 downto 0),
              RdDataZero => Mem_RdDataZero(7 downto 0),
              FirstBit => FirstBit(2 downto 0),
              LastBit => Mem_LastBit(2 downto 0),
              Mem_Addr => Mem_Addr_i(14 downto 0),
              DataOut => DataOutPlayMem(7 downto 0)
              );

  B_ModeSelDecoder: ModeSelDecoder
    port map (
              Mode => Mode(2 downto 0),
              SinglePulseMode => SinglePulseMode,
              InfiniteWindowMode => InfiniteWindowMode,
              WindowedOperationMode => WindowedOperationMode,
              SyncLessOperationMode => SyncLessOperationMode,
              LowFreqGenerationMode => LowFreqGenerationMode,
              PlayMemoryMode => PlayMemoryMode
              );

  B_WrongValuesLogic: WrongValuesLogic
    port map (
              wrongB => wrongB_s,
              wrongHT => wrongHT_s,
              wrongW => wrongW_s,
              BCoarseZero => BCoarseZero,
              HTCoarseZero => HTCoarseZero,
              wValueOne => wValueOne,
              wValueZero => wValueZero,
              Mode => Mode(2 downto 0),
              Clk => Clk,
              Rst => Rst
              );

  B_LimAdder: LimAdderN
    port map (
              A => htValue(63 downto 0),
              B => COne(63 downto 0),
              lim => open,
              O => htValuePlusOne(63 downto 0)
              );

   wrongValue <= ( wrongB_s) or ( wrongHT_s) or ( wrongW_s);

   Stop_seq <= ( Stop) or ( wrongValue);

  process (Start , SyncPulse_i , UseSyncAsStart)
   begin
     case UseSyncAsStart is
       when '0' =>
         Start_i <=  Start;
       when others =>
         Start_i <=  SyncPulse_i;
     end case;
   end process;

  wrongB <= wrongB_s;

  wrongHT <= wrongHT_s;

  wrongW <= wrongW_s;

  process (Start_i , NoStart , DisconnectStart)
   begin
     case DisconnectStart is
       when '0' =>
         Start_seq <=  Start_i;
       when others =>
         Start_seq <=  NoStart;
     end case;
   end process;

      NoStart <= '0';

   Run_i <= ( Run_seq) or ( RunSyncLess) or ( PlayingMem);

  DataOut <= DataOutHTSyncLess when Mode = C_Code_ctuAsVtu_control2_mode_syncLessOperation else
             DataOutLowFreq when Mode = C_Code_ctuAsVtu_control2_mode_lowFreqGeneration else
             DataOutPlayMem when Mode = C_Code_ctuAsVtu_control2_mode_playMemory else
             DataOut_seq;

  SyncPulse <= SyncPulse_i;

  process (htValue , htValuePlusOne , SwitchHTeffective)
   begin
     case SwitchHTeffective is
       when '0' =>
         htValue_effective(63 downto 0) <=  htValue(63 downto 0);
       when others =>
         htValue_effective(63 downto 0) <=  htValuePlusOne(63 downto 0);
     end case;
   end process;

  SwitchHTeffective <= (visual_B_SwitchHTFF_Q);
  SwitchHTeffective_n <= (not visual_B_SwitchHTFF_Q);

  process (Clk , Run_n)
  begin
   if (Run_n = '1') then
      visual_B_SwitchHTFF_Q <= '0';
   elsif (Clk'event and Clk = '1') then

      if (HTSwitchEna = '1') then
        visual_B_SwitchHTFF_Q <= (SwitchHTeffective_n);
      end if;

  end if;
  end process;

  COne(63 downto 0) <= (0=> '1', others => '0');

   HTSwitchEna <= (( OE_2) or ( OE_SyncLess)) and (( htSwitching));

   RstOrStopSeq <= ( Rst) or ( Stop_seq);

   DisconnectStart <= ( SyncLessOperationMode) or ( PlayMemoryMode);

  Run <= Run_i;

  Run_n <= not (Run_i);

  process (DataOut_1 , DataOut_2 , SwitchOutput)
   begin
     case SwitchOutput is
       when '0' =>
         DataOut_seq_i(7 downto 0) <=  DataOut_1(7 downto 0);
       when others =>
         DataOut_seq_i(7 downto 0) <=  DataOut_2(7 downto 0);
     end case;
   end process;

  process (DataOut_1 , DataOut_2 , SwitchtoHT)
   begin
     case SwitchtoHT is
       when '0' =>
         DataIn_2(7 downto 0) <=  DataOut_1(7 downto 0);
       when others =>
         DataIn_2(7 downto 0) <=  DataOut_2(7 downto 0);
     end case;
   end process;

  PulseCount(63 downto 0) <= (visual_B_PulseCounter_cur_state);


  visual_B_PulseCounter_en_state <= visual_B_PulseCounter_next_count
                       when counterEnable = '1'
                       else visual_B_PulseCounter_cur_state;

  visual_B_PulseCounter_next_state <= visual_B_PulseCounter_en_state;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then
    if (counterReset = '1') then
      visual_B_PulseCounter_cur_state <= (others => '0');
   else
       visual_B_PulseCounter_cur_state <= visual_B_PulseCounter_next_state;
   end if;
  end if;
  end process;

  process (visual_B_PulseCounter_cur_state )
  variable plus_minus_one : unsigned(64 - 1 downto 0);

  begin
    plus_minus_one :=
   "0000000000000000000000000000000000000000000000000000000000000001" ;
    visual_B_PulseCounter_next_count <= std_logic_vector(unsigned
  (visual_B_PulseCounter_cur_state) + plus_minus_one);

  end process;

  process (PulseCount , wValue_effective)
   begin

      if ((PulseCount(63 downto 0)) = (wValue_effective(63 downto 0))) then
        WindowDone <= '1';
      else
        if (unsigned((PulseCount(63 downto 0))) < unsigned((wValue_effective(63
   downto 0)))) then
          WindowDone <= '0';
        else
          WindowDone <= '1';
        end if;
      end if;
  end process;

  process (wValue_effective , CZero)
   begin
      if ((wValue_effective(63 downto 0)) = (CZero(63 downto 0))) then
        wValueOne <= '1';
      else
        wValueOne <= '0';
      end if;
  end process;

  process (CZero , wValue)
   begin
      if ((CZero(63 downto 0)) = (wValue(63 downto 0))) then
        wValueZero <= '1';
      else
        wValueZero <= '0';
      end if;
  end process;

      CZero(63 downto 0) <= (others => '0');

   WindowDone_seq <= ( WindowDone) and (not InfiniteWindow);

   counterEnable <= (( OE_1) or ( OE_2)) and (( Run_seq));

   wValueOne_seq <= (( wValueOne) or ( SinglePulseMode)) and ((not
   InfiniteWindow)
  );

   SwitchOutput <= ( SwitchtoHT) or (not Run_seq);

      RestartEnable <= '0';

   InfiniteWindow <= ( InfiniteWindowMode) or ( LowFreqGenerationMode);

  process (DataOut_seq_i , AllZeroOut , RstOrStopSeq)
   begin
     case RstOrStopSeq is
       when '0' =>
         DataOut_seq(7 downto 0) <=  DataOut_seq_i(7 downto 0);
       when others =>
         DataOut_seq(7 downto 0) <=  AllZeroOut(7 downto 0);
     end case;
   end process;

  process (DataOutHTSyncLess , SyncLessStartData , SetStartData)
   begin
     case SetStartData is
       when '0' =>
         DataInHTSyncLess(7 downto 0) <=  DataOutHTSyncLess(7 downto 0);
       when others =>
         DataInHTSyncLess(7 downto 0) <=  SyncLessStartData(7 downto 0);
     end case;
   end process;

  SyncLessStartData(7 downto 0) <= X"01";

   StartSyncLess <= ( Start) and ( SyncLessOperationMode);

  SyncLessDisabled <= not (SyncLessEna);

  ClkValueSwitch <= (visual_B_ClkValueSwitchFF_Q);
  ClkValueSwitch_n <= (not visual_B_ClkValueSwitchFF_Q);

  process (Clk , RstOrStopSeq)
  begin
   if (RstOrStopSeq = '1') then
      visual_B_ClkValueSwitchFF_Q <= '0';
   elsif (Clk'event and Clk = '1') then

      if (DataOutPulse = '1') then
        visual_B_ClkValueSwitchFF_Q <= (ClkValueSwitch_n);
      end if;

  end if;
  end process;

   DataOutPulse <= ( DataOut_seq(7)) or ( DataOut_seq(6)) or ( DataOut_seq(5))
             or ( DataOut_seq(4)) or ( DataOut_seq(3)) or ( DataOut_seq(2))
             or ( DataOut_seq(1)) or ( DataOut_seq(0));

  process (AllZeroOut , AllOneOut , ClkValueSwitch)
   begin
     case ClkValueSwitch is
       when '0' =>
         DataAllEqual(7 downto 0) <=  AllZeroOut(7 downto 0);
       when others =>
         DataAllEqual(7 downto 0) <=  AllOneOut(7 downto 0);
     end case;
   end process;

  process (DataAllEqual , DataFilled , FilledMuxSel)
   begin
     case FilledMuxSel is
       when '0' =>
         DataOutLowFreq(7 downto 0) <=  DataAllEqual(7 downto 0);
       when others =>
         DataOutLowFreq(7 downto 0) <=  DataFilled(7 downto 0);
     end case;
   end process;

      AllZeroOut(7 downto 0) <= (others => '0');

      AllOneOut(7 downto 0) <= (others => '1');

  -- Fill data with ones or zeros after one when DataOutPulse='1'
  process(DataOut_seq, DataOutPulse, ClkValueSwitch)
  begin
    if DataOutPulse = '1' then
      if DataOut_seq(7)='1' then
        if ClkValueSwitch='1' then
          DataFilled <= "10000000";
        else
          DataFilled <= "01111111";
        end if;
      elsif DataOut_seq(6)='1' then
        if ClkValueSwitch='1' then
          DataFilled <= "11000000";
        else
          DataFilled <= "00111111";
        end if;
      elsif DataOut_seq(5)='1' then
        if ClkValueSwitch='1' then
          DataFilled <= "11100000";
        else
          DataFilled <= "00011111";
        end if;
      elsif DataOut_seq(4)='1' then
        if ClkValueSwitch='1' then
          DataFilled <= "11110000";
        else
          DataFilled <= "00001111";
        end if;
      elsif DataOut_seq(3)='1' then
        if ClkValueSwitch='1' then
          DataFilled <= "11111000";
        else
          DataFilled <= "00000111";
        end if;
      elsif DataOut_seq(2)='1' then
        if ClkValueSwitch='1' then
          DataFilled <= "11111100";
        else
          DataFilled <= "00000011";
        end if;
      elsif DataOut_seq(1)='1' then
        if ClkValueSwitch='1' then
          DataFilled <= "11111110";
        else
          DataFilled <= "00000001";
        end if;
      else
        if ClkValueSwitch='1' then
          DataFilled <= "11111111";
        else
          DataFilled <= "00000000";
        end if;
      end if;
    else
      DataFilled <= (others => '0');
    end if;
  end process;

   SyncLessEna <= ( SyncLessOperationMode) and (not RstOrStopSeq);

   FilledMuxSel <= (not RstOrStopSeq) and ( DataOutPulse);

   StartPlayMem <= ( Start_i) and ( PlayMemoryMode);

   StopPlayMem <= (not PlayMemoryMode) or ( RstOrStopSeq);

   RunPlayMem <= (( RunPlayMem_i) or ( StartPlayMem)) and ((not StopPlayMem));

   RunPlayAndSyncPulse <= ( RunPlayMem) and ( SyncPulse_i);

   SetPlayingMem <= ( FirstSyncArrived) and ( OE_1);

   PlayingMem <= (( PlayingMem_i) or ( SetPlayingMem)) and ((not StopPlayMem));

  Mem_RdEna <= RunPlayMem;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then
        PlayingMem_prev <= (PlayingMem);


  end if;
  end process;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then
        Mem_RdData_prev(7 downto 0) <= (Mem_RdData(7 downto 0));


  end if;
  end process;

   FirstOutput <= ( PlayingMem) and (not PlayingMem_prev);

  Mem_Addr(14 downto 0) <= Mem_Addr_i(14 downto 0);

      Mem_AddrZero(14 downto 0) <= (others => '0');

  process (Mem_Addr_i , Mem_AddrZero)
   begin
      if ((Mem_Addr_i(14 downto 0)) = (Mem_AddrZero(14 downto 0))) then
        MemAddrIsZero <= '1';
      else
        MemAddrIsZero <= '0';
      end if;
  end process;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then
        MemAddrIsZeroPrev <= (MemAddrIsZero);


  end if;
  end process;

  process (Clk)
  begin
  if (Clk'event and Clk = '1') then

      if (ReadDataZero = '1') then
        Mem_RdDataZero(7 downto 0) <= (Mem_RdData(7 downto 0));
      end if;

  end if;
  end process;

  FirstBit <= "000" when FirstOutput='1' and DataOut_1(0)='1' else
              "001" when FirstOutput='1' and DataOut_1(1)='1' else
              "010" when FirstOutput='1' and DataOut_1(2)='1' else
              "011" when FirstOutput='1' and DataOut_1(3)='1' else
              "100" when FirstOutput='1' and DataOut_1(4)='1' else
              "101" when FirstOutput='1' and DataOut_1(5)='1' else
              "110" when FirstOutput='1' and DataOut_1(6)='1' else
              "111" when FirstOutput='1' and DataOut_1(7)='1' else
              "000";

   ReadDataZero <= ( MemAddrIsZeroPrev) and ( RunPlayMem);
end vtuCore;
