library ieee;
use ieee.std_logic_1164.all;

use work.wishbone_pkg.all;
use work.trigunit_regs_consts.all;
use work.vtudiag_regs_consts.all;
use work.sim_wishbone.all;
use work.sim_wishbone16.all;

entity tb_vtu is
end tb_vtu;

architecture arch of tb_vtu is
    signal clk_sys : std_logic := '0';
    signal rst_sys : std_logic;

    signal clk_rf : std_logic := '0';
    signal clk_vtu : std_logic := '0';

    signal sync : std_logic := '0';
    signal trig_p : std_logic := '0';
    signal start, stop : std_logic := '0';

    signal wb_in : t_wishbone_slave_in;
    signal wb_out : t_wishbone_slave_out;

    function to_string(v : std_ulogic) return character is
    begin
      case v is
        when 'U' => return 'U';
        when '0' => return '0';
        when '1' => return '1';
        when 'X' => return 'X';
        when 'Z' => return 'Z';
        when 'L' => return 'L';
        when 'H' => return 'H';
        when 'W' => return 'W';
        when '-' => return '-';
      end case;
    end to_string;

    function to_string(v : std_logic_vector) return string is
      alias av : std_logic_vector(1 to v'length) is v;
      variable res : string(av'range);
    begin
      for i in av'range loop
        res (i) := to_string(av (i));
      end loop;
      return res;
    end to_string;
begin
    --  System clock, 62.5 Mhz
    clk_sys <= not clk_sys after 8 ns;
    rst_sys <= '1', '0' after 16 ns;

    --  RF clock, ~200Mhz (5ns)
    --  VTU clock, ~25Mhz (40ns)
    process
      variable cnt : natural := 8;
    begin
      clk_rf <= not clk_rf;
      if cnt = 8 then
        clk_vtu <= not clk_vtu;
        cnt := 0;
      end if;
      cnt := cnt + 1;
      wait for 2.5 ns;
    end process;

    my_inst: entity work.vtu_blk
      port map (
        clk_sys_i => clk_sys,
        rst_sys_i => rst_sys,
        wb_i => wb_in,
        wb_o => wb_out,
        clk_vtu_i => clk_vtu,
        clk_rf_i => clk_rf,
        sync_i => sync,
        trig_o => trig_p,
        start_i => start,
        stop_i => stop
      );

    process
        variable val : std_logic_vector(15 downto 0);
    begin
        sync <= '0';
        start <= '0';

        wait until rst_sys = '0';

        --  Program the diag (500us)
        write16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_TRIGDIAG + ADDR_VTUDIAG_REGS_CONTROL, x"0000");

        --  Program the vtu
        --  Delay between start and the first pulse.
        write64be_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_BVALUEOFFLINE, x"0000_0000_0000_0020");
        --  Delay between the pulses.
        write64be_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_HTVALUEOFFLINE, x"0000_0000_0000_0008");
        --  Number of pulses.
        write64be_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_WVALUEOFFLINE, x"0000_0000_0000_0040");

        --  Windowed mode, enable.
        write16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_CONFIGOFFLINE, x"0021");

        --  Remove reset.
        write16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_CONTROL, x"0000");

        --  Check status.
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_STATUS, val);
        report "status = " & to_string(val);
        assert val (TRIGUNIT_REGS_STATUS_MISSVALID_OFFSET) = '0' severity error;
        assert val (TRIGUNIT_REGS_STATUS_MISSREADY_OFFSET) = '0' severity error;
        assert val (TRIGUNIT_REGS_STATUS_STARTREADY_OFFSET) = '1' severity error;
        assert val (TRIGUNIT_REGS_STATUS_RUNNING_OFFSET) = '0' severity error;
        --  Set to one after reset.
        assert val (TRIGUNIT_REGS_STATUS_WRONGBVALUE_OFFSET) = '1' severity error;
        assert val (TRIGUNIT_REGS_STATUS_WRONGHTVALUE_OFFSET) = '1' severity error;
        assert val (TRIGUNIT_REGS_STATUS_WRONGWVALUE_OFFSET) = '1' severity error;

        --  Check online config is 0.
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_CONFIGONLINE, val);
        report "config online = " & to_string(val);
        assert val = x"0000" severity error;

        --  Start pulse.
        start <= '1';
        wait for 40 ns;
        start <= '0';

        --  Wait until the VTU get the start pulse
        for i in 1 to 7 loop
          wait until rising_edge(clk_vtu);
        end loop;

        --  Check status.
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_STATUS, val);
        report "status = " & to_string(val);
        assert val (TRIGUNIT_REGS_STATUS_MISSVALID_OFFSET) = '0' severity error;
        assert val (TRIGUNIT_REGS_STATUS_MISSREADY_OFFSET) = '0' severity error;
        --  Should be 0 if close enough to the start pulse.
        assert val (TRIGUNIT_REGS_STATUS_STARTREADY_OFFSET) = '0' severity error;
        assert val (TRIGUNIT_REGS_STATUS_RUNNING_OFFSET) = '0' severity error;
        assert val (TRIGUNIT_REGS_STATUS_WRONGBVALUE_OFFSET) = '0' severity error;
        assert val (TRIGUNIT_REGS_STATUS_WRONGHTVALUE_OFFSET) = '0' severity error;
        assert val (TRIGUNIT_REGS_STATUS_WRONGWVALUE_OFFSET) = '0' severity error;

        --  Sync pulse.
        sync <= '1';
        wait for 10 ns;
        sync <= '0';

        --  Check ...
        --  Wait until the VTU get the sync pulse.
        for i in 1 to 3 loop
          wait until rising_edge(clk_vtu);
        end loop;
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_STATUS, val);
        report "status = " & to_string(val);
        assert val (TRIGUNIT_REGS_STATUS_STARTREADY_OFFSET) = '1' severity error;
        assert val (TRIGUNIT_REGS_STATUS_RUNNING_OFFSET) = '1' severity error;

        --  Check online config.
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_CONFIGONLINE, val);
        report "config online = " & to_string(val);
        assert val = x"0020" severity error;

        --  Check online registers
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_WVALUEONLINE + 6, val);
        assert val = x"0040" severity error;
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_WVALUEONLINE + 4, val);
        assert val = x"0000" severity error;
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_WVALUEONLINE + 2, val);
        assert val = x"0000" severity error;
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_WVALUEONLINE + 0, val);
        assert val = x"0000" severity error;
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_HTVALUEONLINE + 6, val);
        assert val = x"0008" severity error;
        read16_pl (clk_sys, wb_in, wb_out, ADDR_TRIGUNIT_REGS_BVALUEONLINE + 6, val);
        assert val = x"0020" severity error;

        wait;
    end process;
end arch;
